import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;
import WeaponsHandler;
import SpellHandler;
import LightHandler;
import XPTable_GUI;
import JumpCalculator_GUI;
import MagicMishapTable_GUI;
import MagicMishapTable;
import Documentation;
import FileSorter;
import GUI;

public class Application extends JFrame implements WindowListener {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private XPTable_GUI xpt_gui;
	private JumpCalculator_GUI jc_gui;
	private MagicMishapTable_GUI mmt_gui;
	private MagicMishapTable mmt;
	private GUI gui;
	private Documentation docs;

	private WeaponsHandler weaponsHandler;
	private GrenadeHandler grenadeHandler;
	private SpellHandler spellHandler;
	private SpellPanel spellPanel;
	private LightHandler lightHandler;

	private JTabbedPane mainPanel;
	private FileSorter fileSorter;

	public final static String weaponsFile = "Weapons.txt";
	public final static String spellFile = "Spells.txt";
	public final static String mishapFile = "MagicMishap.txt";

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public Application() {

		/* Setup the individual panel components. */
		fileSorter = new FileSorter( weaponsFile, "Text" );
		weaponsHandler = new WeaponsHandler( fileSorter );

		fileSorter = new FileSorter( spellFile, "Text" );
		spellHandler = new SpellHandler( fileSorter );
		spellPanel = new SpellPanel( spellHandler );

		fileSorter = new FileSorter( mishapFile, "Text" );
		mmt = new MagicMishapTable( fileSorter );
		mmt_gui = new MagicMishapTable_GUI( mmt );

		lightHandler = new LightHandler();
		grenadeHandler = new GrenadeHandler();
		docs = new Documentation();

		gui = new GUI( this );
		xpt_gui = new XPTable_GUI();
		jc_gui = new JumpCalculator_GUI();

		/* Setup the tabbed pane view. */
		mainPanel = new JTabbedPane( JTabbedPane.TOP );
		mainPanel.setBorder( new EtchedBorder() );
		mainPanel.addTab( "Visual Battlefield", gui );
		mainPanel.addTab( "XP Calculator", xpt_gui );
		mainPanel.addTab( "Jump Calculator", jc_gui );
		mainPanel.addTab( "Magic Mishap", mmt_gui );
		mainPanel.addTab( "Spells", spellPanel );
		mainPanel.addTab( "Documentation", docs );
		mainPanel.setSelectedIndex( 0 );

		this.setTitle( "D&D Integrated Battlefield Services v1.2" );
		this.addWindowListener( this );
		this.getContentPane().setLayout( new BorderLayout() );
		this.getContentPane().add( mainPanel, BorderLayout.CENTER );
		this.setSize( 1024, 768 );
		this.setResizable( false );
		this.setVisible( true );
		gui.focus();
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public WeaponsHandler getWeaponsHandler() {
		return weaponsHandler;
	}

	public SpellHandler getSpellHandler() {
		return spellHandler;
	}

	public LightHandler getLightHandler() {
		return lightHandler;
	}

	public GrenadeHandler getGrenadeHandler() {
		return grenadeHandler;
	}

	public FileSorter getFileSorter() {
		return fileSorter;
	}

	/* Forces a repaint of the frame. */
	public void forcePaint() {
		repaint();
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	public static int generateId() {
		Calendar c = Calendar.getInstance();
		String generator = "" + c.get( Calendar.HOUR_OF_DAY ) +
					c.get( Calendar.MINUTE ) +
					c.get( Calendar.SECOND ) +
					c.get( Calendar.MILLISECOND );
		return  ( new Integer( generator ) ).intValue();
	}


/****************************************************************/
/*	Window Listeners											*/
/****************************************************************/

	public void exitProgram() {
		this.setVisible( false );
		this.dispose();
		System.exit( 0 );
	}

	public void windowActivated( WindowEvent we ) {}
	public void windowDeactivated( WindowEvent we ) {}
	public void windowIconified( WindowEvent we ) {}
	public void windowDeiconified( WindowEvent we ) {}
	public void windowOpened( WindowEvent we ) {}
	public void windowClosed( WindowEvent we ) {}
	public void windowClosing( WindowEvent we ) {
		exitProgram();
	}

/****************************************************************/
/*	Test harness												*/
/****************************************************************/

	public static void main( String[] args ) {
		try {
			new Application();
		}
		catch( Exception e ) {
			e.printStackTrace();
		}
	}
}