import java.util.Vector;
import java.io.Serializable;

public class Character implements Serializable {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private int hp, wounds, ac, rangeIncrement, initiative;
	private int distanceMoved, move, lightRange, moveLeft;
	private int ammo;
	private int meleeMod, rangeMod, dmgMod;
	private float internalInit;
	private String name, weapon, dmg, status, size, light, spell;
	private boolean reachWeapon = false;
	private boolean rangeWeapon = false;
	private boolean grenadeWeapon = false;
	private boolean hasMoveLeft = true;
	private boolean selected = false;

	private Vector feats = new Vector();

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public Character( int h, int a, int i, int z, int m,
				   	  String n, String w, int am, String d,
				   	  String s, String ls, int mm, int rm, int dm,
				   	  String spe ) {
		name = n;
		hp = h;
 		ac = a;
		wounds = z;
		initiative = i;
		internalInit = ( float ) i;
		wounds = 0;
		weapon = w;
		ammo = am;
		move = m/5;
		moveLeft = move;
		dmg = d;
		size = s;
		light = ls;
		meleeMod = mm;
		rangeMod = rm;
		dmgMod = dm;
		spell = spe;
		distanceMoved = 0;
		reachWeapon = checkReachWeapon();
		rangeWeapon = checkRangeWeapon();
		grenadeWeapon = checkGrenade();

		lightRange = checkLightRange( light );
		status = checkStatus();

		if ( rangeWeapon ) {
			rangeIncrement = checkRangeIncrement();
		}
		else {
			rangeIncrement = 0;
		}
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public String getSpell() {
		return spell;
	}

	public int getMeleeMod() {
		return meleeMod;
	}

	public int getRangeMod() {
		return rangeMod;
	}

	public int getDmgMod() {
		return dmgMod;
	}

	public int getDistanceMoved() {
		return distanceMoved;
	}

	public String getLightSource() {
		return light;
	}

	public int getLightRange() {
		return lightRange;
	}

	public String getStatus() {
		return status;
	}

	public String getSize() {
		return size;
	}

	public boolean getRangeWeapon() {
		return rangeWeapon;
	}

	public boolean getReachWeapon() {
		return reachWeapon;
	}

	public int getRangeIncrement() {
		return rangeIncrement;
	}

	public int getMoveLeft() {
		return moveLeft * 5;
	}

	public int getInitiative() {
		return initiative;
	}

	public float getIntInit() {
		return internalInit;
	}

	public int getHP() {
		return hp;
	}

	public int getMove() {
		return move * 5;
	}

	public int getAC() {
		return ac;
	}

	public int getWounds() {
		return wounds;
	}

	public String getName() {
		return name;
	}

	public String getWeapon() {
		return weapon;
	}

	public int getAmmo() {
		return ammo;
	}

	public String getDmg() {
		return dmg;
	}

	public void setSpell( String s ) {
		spell = s;
	}

	public void setAmmo( int i ) {
		ammo = i;
	}

	public void setAmmo( String s ) {
		ammo = ( new Integer( s ) ).intValue();
	}

	public void setInitiative( int i ) {
		initiative = i;
		setIntInit( i );
	}

	public void setIntInit( int i ) {
		internalInit = ( float ) i;
	}

	public void setIntInit( float f ) {
		internalInit = f;
	}

	public void setHP( int i ) {
		hp = i;
	}

	public void setMove( int i ) {
		move = i/5;
	}

	public void setAC( int i ) {
		ac = i;
	}

	public void setWounds( int i ) {
		wounds = i;
	}

	public void setName( String s ) {
		name = s;
	}

	public void setDmg( String s ) {
		dmg = s;
	}

	public void setSize( String s ) {
		size = s;
	}

	public void setWeapon( String s ) {
		weapon = s;
	}

	public void setLightSource( String s ) {
		light = s;
		lightRange = checkLightRange( light );
	}

	public void setDistanceMoved( int i ) {
		distanceMoved = i;
		checkMove();
	}

	public void setMeleeMod( int i ) {
		meleeMod = i;
	}

	public void setRangeMod( int i ) {
		rangeMod = i;
	}

	public void setDmgMod( int i ) {
		dmgMod = i;
	}

	public boolean hasMoveLeft() {
		return hasMoveLeft;
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	public void reCalculateAll() {
		status = checkStatus();
		reachWeapon = checkReachWeapon();
		rangeWeapon = checkRangeWeapon();
		lightRange = checkLightRange( light );
		checkMove();

		if ( rangeWeapon ) {
			rangeIncrement = checkRangeIncrement();
		}
		else {
			rangeIncrement = 0;
		}
	}

/****************************************************************/

	public void checkMove() {
		moveLeft = move - distanceMoved;

		if ( moveLeft <= 0 ) {
			moveLeft = 0;
			hasMoveLeft = false;
		}
		else {
			hasMoveLeft = true;
		}
	}

/****************************************************************/

	public String checkStatus() {
		int offset = hp - wounds;
		if ( offset > 0 ) return "ok";
		else if ( offset <= 0 && offset >= -9 ) return "dying";
		else return "dead";
	}

/****************************************************************/

	private int checkRangeIncrement() {
		String chk = weapon.toLowerCase();
		int rangeIncrement = 0;

		if ( chk.equals( "bow, short" ) ) {
			rangeIncrement = 12;
		}
		else if ( chk.equals( "bow, long" ) ) {
			rangeIncrement = 20;
		}
		else if ( chk.equals( "bow, short, composite" ) ) {
			rangeIncrement = 14;
		}
		else if ( chk.equals( "bow, long, composite" ) ) {
			rangeIncrement = 22;
		}
		else if ( chk.equals( "axe, throwing" ) | chk.equals( "dagger" ) |
			  chk.equals( "shuriken" ) | chk.equals( "trident" ) |
			  chk.equals( "alchemist's fire" ) | chk.equals( "acid, flask" ) |
			  chk.equals( "avienna's fog" ) | chk.equals( "holy water" ) |
			  chk.equals( "stonetar" ) | chk.equals( "tanglefoot bag" ) ) {
			rangeIncrement = 2;
		}
		else if ( chk.equals( "crossbow, light" ) ) {
			rangeIncrement = 16;
		}
		else if ( chk.equals( "crossbow, heavy" ) ) {
			rangeIncrement = 24;
		}
		else if ( chk.equals( "crossbow, hand" ) ) {
			rangeIncrement = 6;
		}
		else if ( chk.equals( "repeater" ) | chk.equals( "crossbow, repeating" ) ) {
			rangeIncrement = 16;
		}
		else if ( chk.equals( "hammer" ) | chk.equals( "hammer, light" ) |
			  chk.equals( "dart" ) | chk.equals( "halfspear" ) |
			  chk.equals( "shortspear" ) | chk.equals( "thunderstone" ) ) {
			rangeIncrement = 4;
		}
		else if ( chk.equals( "sling" ) ) {
			rangeIncrement = 10;
		}
		return rangeIncrement;
	}

/****************************************************************/

	private int checkLightRange( String s ) {
		int lightRange = 0;

		if ( s.equalsIgnoreCase( "candle" ) ) {
			lightRange = 1;
		}
		else if ( s.equalsIgnoreCase( "lamp, common" ) ) {
			lightRange = 3;
		}
		else if ( s.equalsIgnoreCase( "lantern, bullseye" ) | s.equalsIgnoreCase( "daylight" ) ) {
			lightRange = 12;
		}
		else if ( s.equalsIgnoreCase( "lantern, hooded" ) | s.equalsIgnoreCase( "sunrod" ) ) {
			lightRange = 6;
		}
		else if ( s.equalsIgnoreCase( "torch" ) | s.equalsIgnoreCase( "continual flame" ) |
			  s.equalsIgnoreCase( "dancing lights" ) | s.equalsIgnoreCase( "light" ) ) {
			lightRange = 4;
		}
		else {
			lightRange = 0;
		}
		return lightRange;
	}

/****************************************************************/

	private boolean checkRangeWeapon() {
		String chk = weapon.toLowerCase();

		if ( chk.startsWith( "bow" ) | chk.equals( "axe, throwing" )  |
		     chk.equals( "dagger" ) | chk.equals( "dart" ) |
		     chk.equals( "halfspear" ) | chk.startsWith( "crossbow" ) |
		     chk.equals( "hammer, light" ) |  chk.equals( "javelin" ) |
		     chk.equals( "shortspear" ) | chk.equals( "shuriken" ) |
		     chk.equals( "sling" ) | chk.equals( "trident" ) |
		     chk.equals( "thunderstone" ) | chk.equals( "alchemist's fire" ) |
		     chk.equals( "acid, flask" ) | chk.equals( "avienna's fog" ) |
		     chk.equals( "holy water" ) | chk.equals( "stonetar" ) |
		     chk.equals( "tanglefoot bag" ) ) {
			return true;
		}
		else return false;
	}

/****************************************************************/

	private boolean checkGrenade() {
		if ( weapon.equalsIgnoreCase( "acid, flask" ) |
			 weapon.equalsIgnoreCase( "thunderstone" ) |
			 weapon.equalsIgnoreCase( "alchemist's fire" ) |
			 weapon.equalsIgnoreCase( "avienna's fog" ) |
			 weapon.equalsIgnoreCase( "stonetar" ) |
			 weapon.equalsIgnoreCase( "holy water" ) |
			 weapon.equalsIgnoreCase( "tanglefoot bag" ) ) {
			return true;
		}
		else return false;
	}

/****************************************************************/

	private boolean checkReachWeapon() {
		if ( weapon.equalsIgnoreCase( "chain, spiked" ) |
			 weapon.equalsIgnoreCase( "guisarme" ) |
			 weapon.equalsIgnoreCase( "glaive" ) |
		     weapon.equalsIgnoreCase( "lance, heavy" ) |
		     weapon.equalsIgnoreCase( "longspear" ) |
		     weapon.equalsIgnoreCase( "ranseur" ) ) {
			return true;
		}
		else return false;
	}

/****************************************************************/

	/* Follows the PHB formula:
	 * +1 ability modifier for every 2 points in score.
	 */
	public static int getAbilityMod( int i ) {
		int base = -5;
		int modifier = i/2;
		return base + modifier;
	}
}