import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Rectangle;
import java.awt.Point;
import java.util.Vector;
import Marker;
import Grenade;
import GUI;

class DisplayPanel extends JPanel implements MouseListener, MouseMotionListener {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private Point min, max, moveOrigin, moveDest;
	private boolean draw = false;
	private boolean drawBox = false;
	private boolean drawSpell = false;
	private boolean setFriendly = true;
	private boolean drawThreatRange = false;
	private boolean drawMoveRange = false;
	private boolean drawRunRange = false;
	private boolean drawLightRange = false;
	private boolean killLight = false;
	private boolean killGrenade = false;
	private boolean killObstacle = false;
	private boolean moveMarker = false;
	private boolean rectSelected = false;
	private int scale = 0;
	private Vector markerVector = new Vector();
	private Vector lightSources = new Vector();
	private Vector grenadeSources = new Vector();
	private Vector obstacleSources = new Vector();
	private GUI gui;
	private int idToBeMoved = 0;
	private int lightSourceIdToBeMoved = 0;
	private int grenadeIdToBeMoved = 0;
	private int obstacleIdToBeMoved = 0;
	private int moveRate = 1;
	private int displayIndex = 0;
	private Marker markerForDisplay;
	private Rectangle validMoveRange;

	public static int gridSize = 20;
	public static int minX, minY, maxX, maxY;

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public DisplayPanel( GUI g ) {
		gui = g;
		markerVector = g.getMarkerData();
		this.setBackground( Color.white );
		this.setBorder( new EtchedBorder() );
		this.addMouseListener( this );
		setBoundaries();
		draw = true;
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public void refreshData() {
		markerVector = gui.getMarkerData();
		drawBox();
		gui.forcePaint();
	}

/****************************************************************/

	public Vector getMarkerData() {
		return markerVector;
	}

/****************************************************************/

	public void killGrenadeMode() {
		killGrenade = true;
	}

/****************************************************************/

	public void killLightMode() {
		killLight = true;
	}

/****************************************************************/

	public void killObstacleMode() {
		killObstacle = true;
	}

/****************************************************************/

	public void addGrenade( Grenade g ) {
		grenadeSources.add( g );
	}

/****************************************************************/

	public void removeGrenade( Grenade g ) {
		grenadeSources.remove( g );
	}

/****************************************************************/

	public void addLightSource( Light l ) {
		lightSources.add( l );
	}

/****************************************************************/

	public void removeLightSource( Light l ) {
		lightSources.remove( l );
	}

/****************************************************************/

	public void addObstacle( Obstacle o ) {
		obstacleSources.add( o );
	}

/****************************************************************/

	public void removeObstacle( Obstacle o ) {
		obstacleSources.remove( o );
	}

/****************************************************************/

	public int getZoom() {
		return scale;
	}

/****************************************************************/

	public int getGridSize() {
		return gridSize;
	}

/****************************************************************/

	public void resetZoom() {
		scale = 0;
		displayGrid();
		updateAllVectors( scale );
		gui.forcePaint();
	}

/****************************************************************/

	public void setZoom( int i ) {
		scale = i;
		displayGrid();
		updateAllVectors( scale );
		gui.forcePaint();
	}

/****************************************************************/

	/* Convenience method for updating scales for all the vectors. */
	public void updateAllVectors( int scale ) {
		updateMarkerVector( scale );
		updateLightSourceVector( scale );
		updateGrenadeVector( scale );
		updateObstacleVector( scale );
	}

/****************************************************************/

	/** Forces a paint of the display. */
	public void displayGrid() {
		draw = true;
		gui.forcePaint();
	}

/****************************************************************/

	/** Resets the display completely. */
	public void clearAll() {
		resetZoom();
		lightSources = new Vector();
		grenadeSources = new Vector();
		obstacleSources = new Vector();
		drawBox = false;
		drawThreatRange = false;
		drawRunRange = false;
		drawMoveRange = false;
		drawLightRange = false;
		gui.forcePaint();
	}

/****************************************************************/

	/** Highlights the selection box in red. */
	public void drawBox() {
		drawBox = true;
		gui.forcePaint();
	}

/****************************************************************/

	/** Makes the selection box disappear. */
	public void clearBox() {
		drawBox = false;
		gui.forcePaint();
	}

/****************************************************************/

	public void showActiveSpell() {
		drawSpell = true;
		gui.forcePaint();
	}

/****************************************************************/

	public void clearActiveSpell() {
		drawSpell = false;
		gui.forcePaint();
	}

/****************************************************************/

	public void showThreatRange() {
		drawThreatRange = true;
		gui.forcePaint();
	}

/****************************************************************/

	public void showLightRange() {
		drawLightRange = true;
		gui.forcePaint();
	}

/****************************************************************/

	public void clearLightRange() {
		drawLightRange = false;
		gui.forcePaint();
	}

/****************************************************************/

	public void clearThreatRange() {
		drawThreatRange = false;
		gui.forcePaint();
	}

/****************************************************************/

	public void showMoveRange() {
		drawMoveRange = true;
		gui.forcePaint();
	}

/****************************************************************/

	public void clearMoveRange() {
		drawMoveRange = false;
		moveRate = 1;
		gui.forcePaint();
	}

/****************************************************************/

	public void showRunRange() {
		drawRunRange = true;
		gui.forcePaint();
	}

/****************************************************************/

	public void clearRunRange() {
		drawRunRange = false;
		moveRate = 1;
		gui.forcePaint();
	}

/****************************************************************/

	public void paint( Graphics g2 ) {
		try {
			super.paint( g2 );
			paintComponent( g2 );
		}
		catch( InternalError ie ) {
			System.out.println( "Java internal error detected." );
		}
	}

/****************************************************************/

	/** Displays the lightweight component on screen. */
	public void paintComponent( Graphics g2 ) {

		Graphics2D g = ( Graphics2D ) g2 ;

		if ( draw ) {
			makeGridVisible( g );
		}
		if ( drawBox ) {
			makeBoxVisible( g );
		}
		if ( drawThreatRange ) {
			makeRangeVisible( g );
		}
		if ( drawMoveRange ) {
			makeMoveVisible( g );
		}
		if ( drawRunRange ) {
			makeRunVisible( g );
		}
		if ( drawLightRange ) {
			makeLightVisible( g );
		}
		if ( drawSpell ) {
			makeSpellVisible( g );
		}
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	/** Sets the min/max values delimited the boundaries in
	  * which the grid may be drawn in.
	  */
	private void setBoundaries() {
		minX = 0;
		maxX = 1000;
		minY = 0;
		maxY = 700;

		min = new Point( minX, minY );
		max = new Point( maxX, maxY );
	}

/****************************************************************/

	/* Collects the 3 object vectors into a single vector.
	 * The positions are static:
	 * 0 holds the markers.
	 * 1 holds the light sources.
	 * 2 holds the grenade sources.
	 */
	public Vector marshall() {
		Vector result = new Vector();
		result.add( markerVector );
		result.add( lightSources );
		result.add( grenadeSources );
		result.add( obstacleSources );
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Splits a marshalled vector into its 3 component vector.
	 * The reverse of the marshall() method, and the positions
	 * MUST be the same.
	 */
	public void unmarshall( Vector v ) {
		Vector temp = new Vector();

		temp = ( Vector ) v.elementAt( 0 );
		if ( temp != null ) {
			markerVector = temp;
		}
		markerVector.trimToSize();

		temp = ( Vector ) v.elementAt( 1 );
		if ( temp != null ) {
			lightSources = temp;
		}
		lightSources.trimToSize();

		temp = ( Vector ) v.elementAt( 2 );

		if ( temp != null ) {
			grenadeSources = temp;
		}
		grenadeSources.trimToSize();

		temp = ( Vector ) v.elementAt( 3 );

		if ( temp != null ) {
			obstacleSources = temp;
		}
		obstacleSources.trimToSize();
	}

/****************************************************************/

	/* As unmarshall( Vector v ), except it appends the data to
	 * existing vectors.
	 */
	public void appendUnMarshall( Vector v ) {
		Vector temp = new Vector();

		temp = ( Vector ) v.elementAt( 0 );
		if ( temp != null ) {
			markerVector.addAll( temp );
		}
		markerVector.trimToSize();

		temp = ( Vector ) v.elementAt( 1 );
		if ( temp != null ) {
			lightSources.addAll( temp );
		}
		lightSources.trimToSize();

		temp = ( Vector ) v.elementAt( 2 );

		if ( temp != null ) {
			grenadeSources.addAll( temp );
		}
		grenadeSources.trimToSize();

		temp = ( Vector ) v.elementAt( 3 );

		if ( temp != null ) {
			obstacleSources.addAll( temp );
		}
		obstacleSources.trimToSize();
	}

/****************************************************************/

	private void makeLightVisible( Graphics2D g ) {

		if ( markerForDisplay != null ) {
			int lightRange = markerForDisplay.getLightRange();
			String lightSource = markerForDisplay.getLightSource();
			Color lightColor = new Color( Color.yellow.getRed(),
						      			  Color.yellow.getGreen(),
						      			  Color.yellow.getBlue(), 50 );
			int jump = gridSize + scale;
			int sourceX = ( int ) markerForDisplay.getBoundingBox().getX();
			int sourceY = ( int ) markerForDisplay.getBoundingBox().getY();
			int startX, startY, width, height;

			startX = sourceX - ( jump * lightRange );
			startY = sourceY - ( jump * lightRange );
			width = ( jump * lightRange * 2 ) + jump;
			height = ( jump * lightRange * 2 ) + jump;
			Ellipse2D.Double lightShape = new Ellipse2D.Double( startX, startY, width, height );
			g.setColor( lightColor );
			g.fill( lightShape );
		}
	}

/****************************************************************/

	private void makeSpellVisible( Graphics2D g ) {

		if ( markerForDisplay != null ) {
			String activeSpell = markerForDisplay.getActiveSpell();
			int spread = Grenade.calculateSpread( activeSpell );
			Color spellColor = Grenade.calculateColor( activeSpell );

			int jump = gridSize + scale;
			int sourceX = ( int ) markerForDisplay.getBoundingBox().getX();
			int sourceY = ( int ) markerForDisplay.getBoundingBox().getY();
			int startX, startY, width, height;

			startX = sourceX - ( jump * spread );
			startY = sourceY - ( jump * spread );
			width = ( jump * spread * 2 ) + jump;
			height = ( jump * spread * 2 ) + jump;
			Ellipse2D.Double spellShape = new Ellipse2D.Double( startX, startY, width, height );
			g.setColor( spellColor );
			g.fill( spellShape );
		}
	}

/****************************************************************/

	/** Convenience method for drawing all markers and light sources.
	  */
	private void makeBoxVisible( Graphics2D g ) {
		int jump = gridSize + scale;
		Marker tempMarker;
		Light tempSource;
		Grenade tempGrenade;
		Obstacle tempObstacle;
		Rectangle tempRect;
		Ellipse2D.Double tempEllipse;
		Color tempColor;
		String tempName;
		int sourceX, sourceY, startX, startY, width, height;
		int textX, textY, lightRange, splashRange;

		/* Draw all light sources. */
		int size = lightSources.size();
		for ( int j = 0; j < size; j++ ) {
			tempSource = ( Light ) lightSources.elementAt( j );
			tempName = tempSource.getName();
			lightRange = tempSource.getRange();
			tempRect = tempSource.getBoundingBox();

			if ( tempName.equalsIgnoreCase( "Darkness" ) |
				 tempName.equalsIgnoreCase( "Deeper Darkness" ) ) {
				tempColor = new Color( 0, 0, 0, 50 );
			}
			else {
				tempColor = new Color( Color.yellow.getRed(),
						       		   Color.yellow.getGreen(),
						       		   Color.yellow.getBlue(), 50 );
			}

			/* Paint the center of the ellipse first. */
			sourceX = ( int ) tempRect.getX();
			sourceY = ( int ) tempRect.getY();
			tempEllipse = new Ellipse2D.Double( sourceX, sourceY, jump, jump );
			g.setColor( tempColor );
			g.fill( tempEllipse );

			/* Paint the surrounding area, extending to the
			 * max ellipse range.
			 */
			sourceX = ( int ) tempEllipse.getX();
			sourceY = ( int ) tempEllipse.getY();

			startX = sourceX - ( jump * lightRange );
			startY = sourceY - ( jump * lightRange );
			width = ( jump * lightRange * 2 ) + jump;
			height = ( jump * lightRange * 2 ) + jump;
			Ellipse2D.Double lightShape = new Ellipse2D.Double( startX, startY, width, height );
			g.setColor( tempColor );
			g.fill( lightShape );

			/* Paint the text that is associated with this ellipse. */
			textX = ( int ) tempEllipse.getX();
			textY = ( int ) tempEllipse.getCenterY();

			g.setColor( Color.black );
			g.drawString( tempName, textX, textY );
		}

		/* Draw all grenade sources. */
		size = grenadeSources.size();
		for ( int k = 0; k < size; k++ ) {
			tempGrenade = ( Grenade ) grenadeSources.elementAt( k );
			splashRange = tempGrenade.getSpread();
			tempRect = tempGrenade.getBoundingBox();
			tempColor = tempGrenade.getColor();
			tempName = tempGrenade.getName();

			/* Paint the center of the ellipse first. */
			sourceX = ( int ) tempRect.getX();
			sourceY = ( int ) tempRect.getY();
			tempEllipse = new Ellipse2D.Double( sourceX, sourceY, jump, jump );
			g.setColor( tempColor );
			g.fill( tempEllipse );

			/* Paint the surrounding area, extending to the max range of the
			 * ellipse.
			 */
			sourceX = ( int ) tempEllipse.getX();
			sourceY = ( int ) tempEllipse.getY();

			startX = sourceX - ( jump * splashRange );
			startY = sourceY - ( jump * splashRange );
			width = ( jump * splashRange * 2 ) + jump;
			height = ( jump * splashRange * 2 ) + jump;

			Ellipse2D.Double grenadeShape = new Ellipse2D.Double( startX, startY, width, height );
			g.setColor( tempColor );
			g.fill( grenadeShape );

			/* Paint in the text. */
			textX = ( int ) tempEllipse.getX();
			textY = ( int ) tempEllipse.getCenterY();

			g.setColor( Color.black );
			g.drawString( tempName, textX, textY );
		}

		/* Draw all markers. */
		size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {

			tempMarker = ( Marker ) markerVector.elementAt( i );
			tempRect = tempMarker.getBoundingBox();
			tempColor = tempMarker.getMarkerColor();
			tempName = tempMarker.getName();

			/* Paint the heart of the ellipse first. */
			sourceX = ( int ) tempRect.getX();
			sourceY = ( int ) tempRect.getY();
			width = ( int ) tempRect.getWidth();
			height = ( int ) tempRect.getHeight();
			tempEllipse = new Ellipse2D.Double( sourceX, sourceY, width, height );

			g.setColor( tempColor );
			g.fill( tempEllipse );

			/* Outline the marker in black. */
			g.setColor( Color.black );
			g.draw( tempEllipse );

			textX = ( int ) tempEllipse.getX();
			textY = ( int ) tempEllipse.getCenterY();

			/* Write in the name of this marker. */
			g.setColor( Color.black );
			g.drawString( tempName, textX, textY );
		}

		/* Draw all obstacles. */
		size = obstacleSources.size();
		for ( int m = 0; m < size; m++ ) {
			tempObstacle = ( Obstacle ) obstacleSources.elementAt( m );
			tempRect = tempObstacle.getBoundingBox();
			tempColor = tempObstacle.getColor();

			g.setColor( tempColor );
			g.fill( tempRect );
		}

		gui.forcePaint();
	}

/****************************************************************/

	/** Convenience method for drawing the graph grid. */
	private void makeGridVisible( Graphics2D g ) {
		g.setColor( Color.lightGray );

		int jump = gridSize + scale;

		/* Draw horizontal lines. */
		for ( int i = 0; i < maxX; i += jump ) {
			g.drawLine( i, 0, i, maxY );
		}

		/* Draw vertical lines. */
		for ( int j = 0; j < maxY; j += jump ) {
			g.drawLine( 0, j, maxX, j );
		}
		gui.forcePaint();
	}

/****************************************************************/

	private void makeMoveVisible( Graphics2D g ) {
		if ( markerForDisplay != null ) {

			int jump = gridSize + scale;
			Color moveColor = new Color( 0, 0, 125, 50 );
			int moveIncrement = markerForDisplay.getMove() / 5;
			//int moveIncrement = markerForDisplay.getMoveLeft() / 5;
			int moveMod = jump * moveIncrement * moveRate;
			String s = markerForDisplay.getSize();
			int sourceX = ( int ) ( markerForDisplay.getBoundingBox() ).getX();
			int sourceY = ( int ) ( markerForDisplay.getBoundingBox() ).getY();
			int sourceWidth = 0;
			int sourceHeight = 0;
			int totalWidth = 0;
			int totalHeight = 0;
			int startX, startY, endX, width, height;

			if ( s.equalsIgnoreCase( "large, tall" ) | s.equalsIgnoreCase( "lt" ) ) {
				sourceWidth = jump;
				sourceHeight = jump;
			}
			else if ( s.equalsIgnoreCase( "large, long" ) | s.equalsIgnoreCase( "ll" ) ) {
				sourceWidth = jump;
				sourceHeight = jump * 2;
			}
			else if ( s.equalsIgnoreCase( "huge, tall" ) | s.equalsIgnoreCase( "ht" ) ) {
				sourceWidth = jump * 2;
				sourceHeight = jump * 2;
			}
			else if ( s.equalsIgnoreCase( "huge, long" ) | s.equalsIgnoreCase( "hl" ) ) {
				sourceWidth = jump * 2;
				sourceHeight = jump * 4;
			}
			else if ( s.equalsIgnoreCase( "gargantuan, tall" ) | s.equalsIgnoreCase( "gt" ) ) {
				sourceWidth = jump * 4;
				sourceHeight = jump * 4;
			}
			else if ( s.equalsIgnoreCase( "gargantuan, long" ) | s.equalsIgnoreCase( "gl" ) ) {
				sourceWidth = jump * 4;
				sourceHeight = jump * 6;
			}
			else if ( s.equalsIgnoreCase( "colossal, tall" ) | s.equalsIgnoreCase( "ct" ) ) {
				sourceWidth = jump * 8;
				sourceHeight = jump * 8;
			}
			else if ( s.equalsIgnoreCase( "colossal, long" ) | s.equalsIgnoreCase( "cl" ) ) {
				sourceWidth = jump * 8;
				sourceHeight = jump * 16;
			}
			else {
				sourceWidth = jump;
				sourceHeight = jump;
			}

			/* Paint the north area of the marker. */
			startX = sourceX - moveMod;
			startY = sourceY - moveMod;
			endX = sourceX + sourceWidth + moveMod;
			width = Math.abs( startX - endX );
			height = moveMod;
			Rectangle run = new Rectangle( startX, startY, width, height );
			g.setColor( moveColor );
			g.fill( run );
			totalWidth += width;
			totalHeight += height;

			/* Paint the area in-between north and south. */
			startY = sourceY;
			height = sourceHeight;
			run = new Rectangle( startX, startY, width, height );
			g.setColor( moveColor );
			g.fill( run );
			totalHeight += height;

			/* Paint the south area of the marker. */
			startY = sourceY + sourceHeight;
			height = moveMod;
			run = new Rectangle( startX, startY, width, height );
			g.setColor( moveColor );
			g.fill( run );
			totalHeight += height;

			if ( moveMarker ) {
				validMoveRange = new Rectangle( sourceX - moveMod, sourceY - moveMod, totalWidth, totalHeight );
			}
		}
	}

/****************************************************************/

	private void makeRunVisible( Graphics2D g ) {

		if ( markerForDisplay != null ) {

			int jump = gridSize + scale;
			Color moveColor = new Color( 0, 100, 200, 50 );
			int moveIncrement = markerForDisplay.getMove() / 5;
			int jumpRate = moveIncrement * moveRate;
			int sourceX = ( int ) ( markerForDisplay.getBoundingBox() ).getX();
			int sourceY = ( int ) ( markerForDisplay.getBoundingBox() ).getY();

			/* Draw the vertical movement area. */
			int startX = sourceX;
			int startY = sourceY - ( jump * jumpRate );
			int width = ( int ) markerForDisplay.getBoundingBox().getWidth();
			int height = ( jump * jumpRate * 2 ) + ( int ) markerForDisplay.getBoundingBox().getHeight();
			Rectangle reach = new Rectangle( startX, startY, width, height );
			g.setColor( moveColor );
			g.fill( reach );

			/* Draw the horizontal movement area. */
			moveColor = new Color( 100, 0, 200, 50 );
			startX = sourceX - ( jump * jumpRate );
			startY = sourceY;
			width = ( jump * jumpRate * 2 ) + ( int ) markerForDisplay.getBoundingBox().getWidth();
			height = ( int ) markerForDisplay.getBoundingBox().getHeight();
			reach = new Rectangle( startX, startY, width, height );
			g.setColor( moveColor );
			g.fill( reach );

			/* Draw the first diagonal (left to right). */
			moveColor = new Color( 200, 200, 0, 50 );
			int maxSquares = jumpRate * 2 + 1;
			for ( int i = 0; i < maxSquares; i++ ) {
				startX = sourceX - ( jump * jumpRate ) + ( i * jump );
				startY = sourceY - ( jump * jumpRate ) + ( i * jump );
				width = ( int ) markerForDisplay.getBoundingBox().getWidth();
				height = ( int ) markerForDisplay.getBoundingBox().getHeight();
				reach = new Rectangle( startX, startY, width, height );
				g.setColor( moveColor );
				g.fill( reach );
			}

			/* Draw the second diagonal (right to left). */
			moveColor = new Color( 100, 100, 100, 50 );
			for ( int j = 0; j < maxSquares; j++ ) {
				startX = sourceX + ( jump * jumpRate ) - ( j * jump );
				startY = sourceY - ( jump * jumpRate ) + ( j * jump );

				width = ( int ) markerForDisplay.getBoundingBox().getWidth();
				height = ( int ) markerForDisplay.getBoundingBox().getHeight();
				reach = new Rectangle( startX, startY, width, height );
				g.setColor( moveColor );
				g.fill( reach );
			}
		}
	}

/****************************************************************/

	private void makeRangeVisible( Graphics2D g ) {
		if ( markerForDisplay != null ) {

			int jump = gridSize + scale;
			Color reachColor = new Color( 0, 255, 0, 50 );
			Color incrementColor = new Color( 200, 0, 0, 50 );
			Color firstIncrementColor = new Color( 225, 0, 0, 50 );
			Color lastIncrementColor = new Color( 250, 0, 0, 50 );

			String s = markerForDisplay.getSize();
			Rectangle markerBounds = markerForDisplay.getBoundingBox();

			int sourceX = ( int ) ( markerBounds.getX() );
			int sourceY = ( int ) ( markerBounds.getY() );
			int sourceWidth = 0;
			int sourceHeight = 0;

			int startX, startY, endX, width, height, rangeIncrement;
			int naturalReach = 0;

			if ( s.equalsIgnoreCase( "large, tall" ) | s.equalsIgnoreCase( "lt" ) ) {
				naturalReach = jump * 2;
				sourceWidth = jump;
				sourceHeight = jump;
			}
			else if ( s.equalsIgnoreCase( "large, long" ) | s.equalsIgnoreCase( "ll" ) ) {
				naturalReach = jump;
				sourceWidth = jump;
				sourceHeight = jump * 2;
			}
			else if ( s.equalsIgnoreCase( "huge, tall" ) | s.equalsIgnoreCase( "ht" ) ) {
				naturalReach = jump * 3;
				sourceWidth = jump * 2;
				sourceHeight = jump * 2;
			}
			else if ( s.equalsIgnoreCase( "huge, long" ) | s.equalsIgnoreCase( "hl" ) ) {
				naturalReach = jump * 2;
				sourceWidth = jump * 2;
				sourceHeight = jump * 4;
			}
			else if ( s.equalsIgnoreCase( "gargantuan, tall" ) | s.equalsIgnoreCase( "gt" ) ) {
				naturalReach = jump * 4;
				sourceWidth = jump * 4;
				sourceHeight = jump * 4;
				}
			else if ( s.equalsIgnoreCase( "gargantuan, long" ) | s.equalsIgnoreCase( "gl" ) ) {
				naturalReach = jump * 3;
				sourceWidth = jump * 4;
				sourceHeight = jump * 6;
			}
			else if ( s.equalsIgnoreCase( "colossal, tall" ) | s.equalsIgnoreCase( "ct" ) ) {
				naturalReach = jump * 5;
				sourceWidth = jump * 8;
				sourceHeight = jump * 8;
			}
			else if ( s.equalsIgnoreCase( "colossal, long" ) | s.equalsIgnoreCase( "cl" ) ) {
				naturalReach = jump * 3;
				sourceWidth = jump * 8;
				sourceHeight = jump * 16;
			}
			else {
				naturalReach = jump;
				sourceWidth = jump;
				sourceHeight = jump;
			}

			if ( markerForDisplay.getRangeWeapon() ) {
				rangeIncrement = markerForDisplay.getRangeIncrement();
				startX = sourceX - ( jump * rangeIncrement );
				startY = sourceY - ( jump * rangeIncrement );
				width = ( jump * rangeIncrement * 2 ) + jump;
				height = ( jump * rangeIncrement * 2 ) + jump;
				Rectangle reach = new Rectangle( startX, startY, width, height );
				g.setColor( incrementColor );
				g.fill( reach );

				startX = sourceX - ( jump * rangeIncrement * 2 );
				startY = sourceY - ( jump * rangeIncrement * 2 );
				width = ( jump * rangeIncrement * 4 ) + jump;
				height = ( jump * rangeIncrement * 4 ) + jump;
				reach = new Rectangle( startX, startY, width, height );
				g.setColor( firstIncrementColor );
				g.fill( reach );

				startX = sourceX - ( jump * rangeIncrement * 3 );
				startY = sourceY - ( jump * rangeIncrement * 3 );
				width = ( jump * rangeIncrement * 6 ) + jump;
				height = ( jump * rangeIncrement * 6 ) + jump;
				reach = new Rectangle( startX, startY, width, height );
				g.setColor( lastIncrementColor );
				g.fill( reach );
			}
			else {
				if ( !markerForDisplay.getReachWeapon() ) {

					/* Paint the north area of the marker. */
					startX = sourceX - naturalReach;
					startY = sourceY - naturalReach;
					endX = sourceX + sourceWidth + naturalReach;
					width = Math.abs( startX - endX );
					height = naturalReach;
					Rectangle reach = new Rectangle( startX, startY, width, height );
					g.setColor( reachColor );
					g.fill( reach );

					/* Paint the area in-between north and south. */
					startY = sourceY;
					height = sourceHeight;
					reach = new Rectangle( startX, startY, width, height );
					g.setColor( reachColor );
					g.fill( reach );

					/* Paint the south area of the marker. */
					startY = sourceY + sourceHeight;
					height = naturalReach;
					reach = new Rectangle( startX, startY, width, height );
					g.setColor( reachColor );
					g.fill( reach );
				}
				else {
					/* A reach weapon grants an extra square of threat. */
					startX = sourceX - naturalReach - jump;
					startY = sourceY - naturalReach - jump;
					endX = sourceX + sourceWidth + naturalReach + jump;
					width = Math.abs( startX - endX );
					height = naturalReach + jump;
					Rectangle reach = new Rectangle( startX, startY, width, height );
					g.setColor( reachColor );
					g.fill( reach );

					startY = sourceY;
					height = sourceHeight;
					reach = new Rectangle( startX, startY, width, height );
					g.setColor( reachColor );
					g.fill( reach );

					startY = sourceY + sourceHeight;
					height = naturalReach + jump;
					reach = new Rectangle( startX, startY, width, height );
					g.setColor( reachColor );
					g.fill( reach );
				}
			}
		}
	}

/****************************************************************/

	/** Returns a rectangle covering a grid box. */
	private Rectangle createSelectionRectangle( Point one, Point two ) {
		Point p = new Point( Math.min( ( int ) one.getX(), ( int ) two.getX() ),
				     Math.min( ( int ) one.getY(), ( int ) two.getY() ) );
		int width = ( int ) Math.abs( one.getX() - two.getX() );
		int height = ( int ) Math.abs( one.getY() - two.getY() );
		return new Rectangle( ( int ) p.getX(), ( int ) p.getY(), width, height );
	}

/****************************************************************/

	/** Finds the square in which the mouse was clicked in. */
	private Rectangle getClickRectangle( Point p ) {
		int jump = gridSize + scale;
		int xcoord = ( int ) p.getX();
		int ycoord = ( int ) p.getY();
		Point rectStart = new Point( xcoord - xcoord%jump,
					     ycoord - ycoord%jump );
		Point rectEnd = new Point( xcoord - xcoord%jump + jump, ycoord - ycoord%jump + jump );
		Rectangle target = createSelectionRectangle( rectStart, rectEnd );
		return target;
	}

/****************************************************************

	/* Returns true iff a point clicked on contains a marker. */
	public boolean markerAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			Marker temp = ( Marker ) markerVector.elementAt( i );
			Rectangle target = temp.getBoundingBox();

			if ( target.contains( markerBounds ) ) {
				return true;
			}
		}
		return false;
	}

/****************************************************************/

	/* Returns true iff a point clicked on contains a light source. */
	public boolean lightSourceAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = lightSources.size();
		for ( int i = 0; i < size; i++ ) {
			Light temp = ( Light ) lightSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();

			if ( target.contains( markerBounds ) ) {
				return true;
			}
		}
		return false;
	}

/****************************************************************/

	/* Returns the light source iff the point clicked on has a light source. */
	private Light getLightSourceAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = lightSources.size();
		for ( int i = 0; i < size; i++ ) {
			Light temp = ( Light ) lightSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp;
			}
		}
		return null;
	}

/****************************************************************/

	/* Returns true iff a point clicked on contains a grenade source. */
	public boolean grenadeAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = grenadeSources.size();
		for ( int i = 0; i < size; i++ ) {
			Grenade temp = ( Grenade ) grenadeSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();

			if ( target.contains( markerBounds ) ) {
				return true;
			}
		}
		return false;
	}

/****************************************************************/

	/* Returns true iff a point clicked on contains an obstacle. */
	public boolean obstacleAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = obstacleSources.size();
		for ( int i = 0; i < size; i++ ) {
			Obstacle temp = ( Obstacle ) obstacleSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();

			if ( target.contains( markerBounds ) ) {
				return true;
			}
		}
		return false;
	}

/****************************************************************/

	/* Returns the grenade marker iff the point clicked on has a grenade. */
	public Grenade getGrenadeAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = grenadeSources.size();
		for ( int i = 0; i < size; i++ ) {
			Grenade temp = ( Grenade ) grenadeSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();

			if ( target.contains( markerBounds ) ) {
				return temp;
			}
		}
		return null;
	}

/****************************************************************/

	/* Returns the marker iff the point clicked on has a marker. */
	private Marker getMarkerAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			Marker temp = ( Marker ) markerVector.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp;
			}
		}
		return null;
	}

/****************************************************************/

	/* Returns the obstacle iff the point clicked on has a marker. */
	private Obstacle getObstacleAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = obstacleSources.size();
		for ( int i = 0; i < size; i++ ) {
			Obstacle temp = ( Obstacle ) obstacleSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp;
			}
		}
		return null;
	}


/****************************************************************/

	/* Returns the marker id iff the point clicked on has a marker. */
	private int markerIdAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			Marker temp = ( Marker ) markerVector.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp.getId();
			}
		}
		return -1;
	}

/****************************************************************/

	/* Returns the light source id iff the point clicked on has a light source. */
	private int lightSourceIdAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = lightSources.size();
		for ( int i = 0; i < size; i++ ) {
			Light temp = ( Light ) lightSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp.getId();
			}
		}
		return -1;
	}

/****************************************************************/

	/* Returns the grenade id iff the point clicked on has a grenade source. */
	private int grenadeIdAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = grenadeSources.size();
		for ( int i = 0; i < size; i++ ) {
			Grenade temp = ( Grenade ) grenadeSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp.getId();
			}
		}
		return -1;
	}

/****************************************************************/

	/* Returns the obstacle id iff the point clicked on has an obstacle. */
	private int obstacleIdAt( Point p ) {
		Rectangle markerBounds = getClickRectangle( p );

		int size = obstacleSources.size();
		for ( int i = 0; i < size; i++ ) {
			Obstacle temp = ( Obstacle ) obstacleSources.elementAt( i );
			Rectangle target = temp.getBoundingBox();
			if ( target.contains( markerBounds ) ) {
				return temp.getId();
			}
		}
		return -1;
	}

/****************************************************************/

	/* Returns the marker with the given id. */
	private Marker markerWithId( int target ) {
		Marker temp;
		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			temp = ( Marker ) markerVector.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return temp;
		}
		return null;
	}

/****************************************************************/

	/* Returns the light source with the given id. */
	private Light lightSourceWithId( int target ) {
		Light temp;
		int size = lightSources.size();
		for ( int i = 0; i < size; i++ ) {
			temp = ( Light ) lightSources.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return temp;
		}
		return null;
	}

/****************************************************************/

	/* Returns the grenade with the given id. */
	private Grenade grenadeWithId( int target ) {
		int size = grenadeSources.size();
		for ( int i = 0; i < size; i++ ) {
			Grenade temp = ( Grenade ) grenadeSources.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return temp;
		}
		return null;
	}

/****************************************************************/

	/* Returns the obstacle with the given id. */
	private Obstacle obstacleWithId( int target ) {
		int size = obstacleSources.size();
		for ( int i = 0; i < size; i++ ) {
			Obstacle temp = ( Obstacle ) obstacleSources.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return temp;
		}
		return null;
	}

/****************************************************************/

	/* Returns the array index of the marker with the given id. */
	private int indexForMarker( int target ) {
		Marker temp;
		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			temp = ( Marker ) markerVector.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return i;
		}
		return -1;
	}

/****************************************************************/

	/* Returns the array index of the light source with the given id. */
	private int indexForLightSource( int target ) {
		Light temp;
		int size = lightSources.size();
		for ( int i = 0; i < size; i++ ) {
			temp = ( Light ) lightSources.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return i;
		}
		return -1;
	}

/****************************************************************/

	/* Returns the array index of the grenade with the given id. */
	private int indexForGrenade( int target ) {
		int size = grenadeSources.size();
		for ( int i = 0; i < size; i++ ) {
			Grenade temp = ( Grenade ) grenadeSources.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return i;
		}
		return -1;
	}

/****************************************************************/

	/* Returns the array index of the obstacle with the given id. */
	private int indexForObstacle( int target ) {
		int size = obstacleSources.size();
		for ( int i = 0; i < size; i++ ) {
			Obstacle temp = ( Obstacle ) obstacleSources.elementAt( i );
			int source = temp.getId();
			if ( source == target ) return i;
		}
		return -1;

	}

/****************************************************************/

	/* Calculates the bounding rectangle's coordinates for placing a marker.
	 */
	private Rectangle calculateMarker( Point p, String s ) {

		int jump = gridSize + scale;
		double xcoord = p.getX();
		double ycoord = p.getY();
		double boundingX = xcoord - xcoord%jump;
		double boundingY = ycoord - ycoord%jump;
		int width, height;

		if ( s.equalsIgnoreCase( "large, tall" ) | s.equalsIgnoreCase( "lt" ) ) {
			width = jump;
			height = jump;
		}
		else if ( s.equalsIgnoreCase( "large, long" ) | s.equalsIgnoreCase( "ll" ) ) {
			width = jump;
			height = jump * 2;
		}
		else if ( s.equalsIgnoreCase( "huge, tall" ) | s.equalsIgnoreCase( "ht" ) ) {
			width = jump * 2;
			height = jump * 2;
		}
		else if ( s.equalsIgnoreCase( "huge, long" ) | s.equalsIgnoreCase( "hl" ) ) {
			width = jump * 2;
			height = jump * 4;
		}
		else if ( s.equalsIgnoreCase( "gargantuan, tall" ) | s.equalsIgnoreCase( "gt" ) ) {
			width = jump * 4;
			height = jump * 4;
		}
		else if ( s.equalsIgnoreCase( "gargantuan, long" ) | s.equalsIgnoreCase( "gl" ) ) {
			width = jump * 4;
			height = jump * 6;
		}
		else if ( s.equalsIgnoreCase( "colossal, tall" ) | s.equalsIgnoreCase( "ct" ) ) {
			width = jump * 8;
			height = jump * 8;
		}
		else if ( s.equalsIgnoreCase( "colossal, long" ) | s.equalsIgnoreCase( "cl" ) ) {
			width = jump * 8;
			height = jump * 16;
		}
		else {
			width = jump;
			height = jump;
		}

		Rectangle marker = new Rectangle( ( int ) boundingX, ( int ) boundingY, width, height );
		return marker;
	}

/****************************************************************/

	/* Calculates the default bounding box for a marker. */
	public Marker calculateBoundingBox( Point p, Marker m ) {
		String size = m.getSize();
		Rectangle markerBounds = calculateMarker( p, size );
		m.setBoundingBox( markerBounds );
		return m;
	}

/****************************************************************/

	/* Calculates the default bounding box for a light source. */
	public Light calculateBoundingBox( Point p, Light ls ) {
		int jump = gridSize + scale;
		double xcoord = p.getX();
		double ycoord = p.getY();
		double boundingX = xcoord - xcoord%jump;
		double boundingY = ycoord - ycoord%jump;
		double width = ( double ) jump;
		double height = width;

		Rectangle sourceBounds = new Rectangle( ( int ) boundingX, ( int ) boundingY, ( int ) width, ( int ) height );
		ls.setBoundingBox( sourceBounds );

		return ls;
	}

/****************************************************************/

	/* Calculates the default bounding box and ellipse for a grenade. */
	public Grenade calculateBoundingBox( Point p, Grenade g ) {
		int jump = gridSize + scale;
		double xcoord = p.getX();
		double ycoord = p.getY();
		double boundingX = xcoord - xcoord%jump;
		double boundingY = ycoord - ycoord%jump;
		double width = ( double ) jump;
		double height = width;

		Rectangle sourceBounds = new Rectangle( ( int ) boundingX, ( int ) boundingY, ( int ) width, ( int ) height );
		g.setBoundingBox( sourceBounds );

		return g;
	}

/****************************************************************/

	/* Calculates the bounding box for an obstacle. */
	public Obstacle calculateBoundingBox( Point p, Obstacle o ) {
		o.setBoundingBox( getClickRectangle( p ) );
		return o;
	}

/****************************************************************/

	/* Updates all entries in the markerVector to the specified scale. */
	private void updateMarkerVector( int scale ) {
		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			Marker temp = ( Marker ) markerVector.elementAt( i );
			Rectangle bbTemp = temp.getBoundingBox();
			int sourceX = bbTemp.x;
			int sourceY = bbTemp.y;
			Point source = new Point( sourceX, sourceY );
			temp = calculateBoundingBox( source, temp );
		}
	}

/****************************************************************/

	/* Updates all entries in the light sources vector to the specified scale. */
	private void updateLightSourceVector( int scale ) {
		int size = lightSources.size();
		for ( int i = 0; i < size; i++ ) {
			Light temp = ( Light ) lightSources.elementAt( i );
			Rectangle bbTemp = temp.getBoundingBox();
			int sourceX = bbTemp.x;
			int sourceY = bbTemp.y;
			Point source = new Point( sourceX, sourceY );
			temp = calculateBoundingBox( source, temp );
		}
	}

/****************************************************************/

	/* Updates all entries in the grenades vector to the specified scale. */
	private void updateGrenadeVector( int scale ) {
		int size = grenadeSources.size();
		for ( int i = 0; i < size; i++ ) {
			Grenade temp = ( Grenade ) grenadeSources.elementAt( i );
			Rectangle bbTemp = temp.getBoundingBox();
			int sourceX = bbTemp.x;
			int sourceY = bbTemp.y;
			Point source = new Point( sourceX, sourceY );
			temp = calculateBoundingBox( source, temp );
		}
	}

/****************************************************************/

	/* Updates all entries in the obstacles vector to the specified scale. */
	private void updateObstacleVector( int scale ) {
		int size = obstacleSources.size();
		for ( int i = 0; i < size; i++ ) {
			Obstacle temp = ( Obstacle ) obstacleSources.elementAt( i );
			Rectangle bbTemp = temp.getBoundingBox();
			int sourceX = bbTemp.x;
			int sourceY = bbTemp.y;
			Point source = new Point( sourceX, sourceY );
			temp = calculateBoundingBox( source, temp );
		}
	}

/****************************************************************/

	/* Convenience method for automatically removing dead markers,
	 * updating the player list, etc.
	 */
	public void resetPlayerStatus() {
		int size = markerVector.size();
		for ( int i = 0; i < size; i++ ) {
			Marker temp = ( Marker ) markerVector.elementAt( i );
			temp.reCalculateAll();
			String status = temp.getStatus();

			if ( status.equalsIgnoreCase( "dead" ) ) {
				gui.deleteListItem( i );
				gui.emptyTextFields();
			}
		}
	}

/****************************************************************/

	/* Returns the distance between two points. */
	private double distanceBetween( Point p, Point q ) {
		double px = p.getX();
		double py = p.getY();
		double qx = q.getX();
		double qy = q.getY();
		double distX = Math.abs( px - qx );
		double distY = Math.abs( py - qy );

		double moveX = Math.pow( distX, ( double ) 2 );
		double moveY = Math.pow( distY, ( double ) 2 );

		return Math.sqrt( moveX + moveY );
	}

/****************************************************************/

	/* Convenience method for calculating how many squares of
	 * intervening distance there are between two points.
	 */
	private int distanceInSquares( Point p, Point q ) {
		int jump = gridSize + scale;
		int dist = ( int ) distanceBetween( p, q );
		int result = dist / jump;
		return result;
	}

/****************************************************************/
/*	Event handlers						*/
/****************************************************************/

	public void mouseDragged( MouseEvent me ) {}
	public void mouseMoved( MouseEvent me ) {}
	public void mouseEntered( MouseEvent me ) {}
	public void mouseExited( MouseEvent me ) {}

/****************************************************************/

	public void mousePressed( MouseEvent me ) {

		Point p = me.getPoint();

		clearThreatRange();
		clearLightRange();
		clearActiveSpell();

		if ( !me.isMetaDown() & !me.isAltDown() & !me.isControlDown() & !me.isShiftDown() ) {

			/* If a marker is found at the mouse click location,
			 * pick it up and load it into memory for movement.
			 */
			if ( markerAt( p ) & lightSourceAt( p ) & grenadeAt( p ) & obstacleAt( p ) |
				 markerAt( p ) & lightSourceAt( p ) & !grenadeAt( p ) & obstacleAt( p ) |
				 markerAt( p ) & !lightSourceAt( p ) & grenadeAt( p ) & obstacleAt( p ) |
				 markerAt( p ) & !lightSourceAt( p ) & !grenadeAt( p ) & obstacleAt( p ) |
				 markerAt( p ) & lightSourceAt( p ) & grenadeAt( p ) & !obstacleAt( p ) |
				 markerAt( p ) & !lightSourceAt( p ) & grenadeAt( p ) & !obstacleAt( p ) |
				 markerAt( p ) & lightSourceAt( p ) & !grenadeAt( p ) & !obstacleAt( p ) |
				 markerAt( p ) & !lightSourceAt( p ) & !grenadeAt( p ) & !obstacleAt( p ) ) {

				/* In cases when a marker overlaps with a light source or grenade,
				 * or both, priority for movement is always given to the marker.
				 */
				grenadeIdToBeMoved = -1;
				lightSourceIdToBeMoved = -1;
				obstacleIdToBeMoved = -1;
				moveOrigin = p;
				idToBeMoved = markerIdAt( p );
				int displayIndex = indexForMarker( idToBeMoved );
				gui.showListItem( displayIndex );
			}
			else if ( !markerAt( p ) & lightSourceAt( p ) & grenadeAt( p ) & obstacleAt( p ) |
					  !markerAt( p ) & lightSourceAt( p ) & !grenadeAt( p ) & obstacleAt( p ) |
					  !markerAt( p ) & lightSourceAt( p ) & grenadeAt( p ) & !obstacleAt( p ) |
					  !markerAt( p ) & lightSourceAt( p ) & !grenadeAt( p ) & !obstacleAt( p ) ) {
				/* In cases where a light source is involved in an overlap, but no markers,
				 * priority for movement is always given to the light source.
				 */
				idToBeMoved = -1;
				grenadeIdToBeMoved = -1;
				obstacleIdToBeMoved = -1;
				lightSourceIdToBeMoved = lightSourceIdAt( p );
			}
			else if ( !markerAt( p ) & !lightSourceAt( p ) & grenadeAt( p ) & obstacleAt( p ) |
					  !markerAt( p ) & !lightSourceAt( p ) & grenadeAt( p ) & !obstacleAt( p ) ) {
				/* In cases where there is a grenade involved in an overlap, but no markers or lights,
				 * priority for movement is always given to the grenade.
				 */
				idToBeMoved = -1;
				lightSourceIdToBeMoved = -1;
				obstacleIdToBeMoved = -1;
				grenadeIdToBeMoved = grenadeIdAt( p );
			}
			else if ( !markerAt( p ) & !lightSourceAt( p ) & !grenadeAt( p ) & obstacleAt( p ) ) {
				/* Lastly, once all possibilities are eliminated, there are only 2 cases left:
				 * obstacles only and no markers at all.
				 *
				 * We can now safely shift only the obstacle marker without fear of moving
				 * anything else by accident.
				 */
				idToBeMoved = -1;
				lightSourceIdToBeMoved = -1;
				grenadeIdToBeMoved = -1;
				obstacleIdToBeMoved = obstacleIdAt( p );
			}
			else {
				/* Nothing available at the click location. */
				grenadeIdToBeMoved = -1;
				lightSourceIdToBeMoved = -1;
				idToBeMoved = -1;
				obstacleIdToBeMoved = -1;
			}
		}
	}

/****************************************************************/

	public void mouseReleased( MouseEvent me ) {

		Point p = me.getPoint();

		clearMoveRange();
		clearRunRange();
		clearThreatRange();
		clearLightRange();
		clearActiveSpell();

		if ( !me.isMetaDown() & !me.isAltDown() & !me.isControlDown() & !me.isShiftDown() ) {

			/* Release a marker iff there is one in memory. */
			if ( idToBeMoved != -1 ) {
				Marker moved = markerWithId( idToBeMoved );
				moveDest = p;

				int squaresMoved = distanceInSquares( moveOrigin, moveDest );

				/* Check if there's a marker in memory first. */
				if ( moved != null ) {

					/* Check if a valid range of movement has been
					 * identified.
					 */
					if ( validMoveRange != null ) {

						/* Check if the mouse is being released within
						 * this marker's valid range of movement.
						 */
						if ( validMoveRange.contains( p ) ) {
							if ( !obstacleAt( p ) ) {
								moved = calculateBoundingBox( p, moved );
								moved.setDistanceMoved( squaresMoved );
							}
						}
						validMoveRange = null;
					}
					else {

						/* No valid range of movement has been computed,
						 * which means that this is a free move.
						 */
						if ( !obstacleAt( p ) ) {
							moved = calculateBoundingBox( p, moved );
						}
					}
				}
			}
			else if ( lightSourceIdToBeMoved != -1 ) {
				Light moved = lightSourceWithId( lightSourceIdToBeMoved );

				if ( moved != null ) {
					if ( !obstacleAt( p ) ) {
						moved = calculateBoundingBox( p, moved );
					}
				}
			}
			else if ( grenadeIdToBeMoved != -1 ) {
				Grenade moved = grenadeWithId( grenadeIdToBeMoved );

				if ( moved != null ) {
					if ( !obstacleAt( p ) ) {
						moved = calculateBoundingBox( p, moved );
					}
				}
			}
			else if ( obstacleIdToBeMoved != -1 ) {
				Obstacle moved = obstacleWithId( obstacleIdToBeMoved );

				if ( !grenadeAt( p ) | !lightSourceAt( p ) | ! markerAt( p ) ) {
					moved = calculateBoundingBox( p, moved );
				}
			}
		}
		gui.forcePaint();
	}

/****************************************************************/

	public void mouseClicked( MouseEvent me ) {

		Point p = me.getPoint();
		int numberOfClicks = me.getClickCount();

		clearThreatRange();
		clearLightRange();
		clearActiveSpell();

		if ( killLight | killGrenade | killObstacle ) {
			if ( lightSourceAt( p ) ) {
				removeLightSource( getLightSourceAt( p ) );
			}

			if ( grenadeAt( p ) ) {
				removeGrenade( getGrenadeAt( p ) );
			}

			if ( obstacleAt( p ) ) {
				removeObstacle( getObstacleAt( p ) );
			}

			killLight = false;
			killGrenade = false;
			killObstacle = false;
		}
		else {
			if ( ( markerAt( p ) & !lightSourceAt( p ) ) | ( !markerAt( p ) & lightSourceAt( p ) ) ) {

				markerForDisplay = getMarkerAt( p );

				/* Handle right-clicking first. */
				if ( me.isMetaDown() ) {
					if ( numberOfClicks == 2 ) {
						/* Double-right-click gives light range. */
						showLightRange();
					}
					else {
						/* Single-right-click gives threat range. */
						showThreatRange();
					}
				}
				/* Now handle left-clicks. */
				else if ( me.isAltDown() ) {
					/* Alt-left-click gives normal movement. */
					moveMarker = true;
					moveRate = 1;
					showMoveRange();
				}
				else if ( me.isShiftDown() ) {
					/* Shift-left-click gives hustle movement. */
					moveMarker = true;
					moveRate = 2;
					showMoveRange();
				}
				else if ( me.isControlDown() ) {
					/* Control-left-click gives run movement. */
					moveRate = 3;
					showRunRange();
				}
				else {
					/* Handle left clicks with no modifiers. */
					if ( numberOfClicks == 2 ) {
						/* Double-left-clicking shows an active spell on the
						 * marker.
						 */
						 showActiveSpell();
					}
					else {

					}
				}
			}
			else {
				gui.newPlayerDoClick();
			}
		}
		gui.forcePaint();
	}

/****************************************************************/
}
