import javax.swing.*;
import javax.swing.JFileChooser;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.Vector;
import AcceptedFileFilters;
import FileSorter;
import GUI;

public class FileManagementPanel extends JPanel implements ActionListener {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private FileSorter fs;
	private GUI gui;
	private AcceptedFileFilters objFilter, txtFilter;

	private JButton save, load, merge;
	private JFileChooser jfc;
	private File currentDirectory = new File( "." );

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public FileManagementPanel( FileSorter source, GUI g ) {

		/* Setup pointers. */
		fs = source;
		gui = g;

		/* Setup accepted file filters. */
		objFilter = new AcceptedFileFilters( "obj", "Internal Object File" );
		txtFilter = new AcceptedFileFilters( "txt", "Text File" );
		/* Initialize components. */
		save = new JButton( "Save" );
		load = new JButton( "Load" );
		merge = new JButton( "Merge on-screen data" );
		jfc = new JFileChooser( currentDirectory );
		jfc.setMultiSelectionEnabled( false );
		jfc.addChoosableFileFilter( objFilter );
		jfc.addChoosableFileFilter( txtFilter );

		/* Register action listeners for the buttons. */
		save.addActionListener( this );
		load.addActionListener( this );
		merge.addActionListener( this );

		/* Setup look & feel for this panel. */
		//this.setBorder( new EtchedBorder() );
		this.setLayout( new FlowLayout() );

		/* Finish the panel. */
		this.add( save );
		this.add( load );
		this.add( merge );
	}

/****************************************************************/
/*	Action Listener												*/
/****************************************************************/

	public void actionPerformed( ActionEvent ae ) {

		int retval;
		String currentEntry;
		File currentFile;
		Vector data = new Vector();

		/* Determine which type of dialog box to show. */
		if ( ae.getSource() == save ) {
			retval = jfc.showSaveDialog( null );

			/* Deal with each type of return value. */
			if ( retval == JFileChooser.APPROVE_OPTION ) {
				currentFile = jfc.getSelectedFile();

				if ( currentFile != null ) {
					currentEntry = currentFile.getName();
					data = gui.getMarshalledData();
					fs.write( currentEntry, data, "Object" );
				}
			}
			else if ( retval == JFileChooser.CANCEL_OPTION ) {

			}
			else if ( retval == JFileChooser.ERROR_OPTION ) {
				JOptionPane.showMessageDialog( null, "An unexpected error occurred. The file was not saved." );
			}
			else {
				JOptionPane.showMessageDialog( null, "An unknown operation was attempted." );
			}
		}
		else if ( ae.getSource() == merge ) {
			retval = jfc.showOpenDialog( null );

			if ( retval == JFileChooser.APPROVE_OPTION ) {
				currentFile = jfc.getSelectedFile();

				if ( currentFile != null ) {
					currentEntry = currentFile.getName();
					data = fs.read( currentEntry, "Object" );
					gui.appendUnMarshalData( data );
				}
			}
			else if ( retval == JFileChooser.CANCEL_OPTION ) {

			}
			else if ( retval == JFileChooser.ERROR_OPTION ) {
				JOptionPane.showMessageDialog( null, "An unexpected error occurred. The file was not loaded." );
			}
			else {
				JOptionPane.showMessageDialog( null, "An unknown operation was attempted." );
			}
		}
		else {
			retval = jfc.showOpenDialog( null );

			/* Deal with each type of return value. */
			if ( retval == JFileChooser.APPROVE_OPTION ) {
				currentFile = jfc.getSelectedFile();

				if ( currentFile != null ) {
					currentEntry = currentFile.getName();
					data = fs.read( currentEntry, "Object" );
					gui.unMarshalData( data );
				}
			}
			else if ( retval == JFileChooser.CANCEL_OPTION ) {

			}
			else if ( retval == JFileChooser.ERROR_OPTION ) {
				JOptionPane.showMessageDialog( null, "An unexpected error occurred. The file was not loaded." );
			}
			else {
				JOptionPane.showMessageDialog( null, "An unknown operation was attempted." );
			}
		}
	}
}