/* Takes a text file of entries and sorts them alphabetically. */
import java.io.*;
import java.util.Vector;
import java.util.Arrays;
import java.awt.Point;
import java.awt.Rectangle;

public class FileSorter {

/****************************************************************/
/*	Declarations						*/
/****************************************************************/

	File input;
	FileReader inputStream;
	FileWriter outputStream;
	FileInputStream fis;
	FileOutputStream fos;
	ObjectInputStream ois;
	ObjectOutputStream oos;
	BufferedReader inputBuffer;
	Vector fileEntries = new Vector();

/****************************************************************/
/*	Constructors						*/
/****************************************************************/

	public FileSorter( String filename, String fileType ) {
		try {
			input = new File( filename );

			if ( input.exists() ) {
				if ( fileType.equalsIgnoreCase( "Object" ) ) {
					fis = new FileInputStream( filename );
					ois = new ObjectInputStream( fis );
				}
				else {
					inputStream = new FileReader( input );
					inputBuffer = new BufferedReader( inputStream );
				}
			}
			else {
				System.out.println( filename + " not found." );
				System.out.println( "Creating " + filename );
				input.createNewFile();
			}
		}
		catch( Exception e ) {
			e.printStackTrace();
		}
	}

/****************************************************************/
/*	Accessors						*/
/****************************************************************/

	public void setFile( String filename ) {
		input = new File( filename );
	}

	public File getFile() {
		return input;
	}

	public Vector getFileEntries() {
		return fileEntries;
	}

	public void setFileEntries( Vector v ) {
		fileEntries = v;
	}

	public String getEntryAt( int index ) {
		return ( String ) fileEntries.elementAt( index );
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	/* Returns a vector sorted alphabetically by first character. */
	private Vector sortVector( Vector v ) {
		Object[] temp = v.toArray();
		Arrays.sort( temp );
		Vector result = new Vector();

		int length = temp.length;
		for ( int i = 0; i < length; i++ ) {
			result.add( ( String ) temp[ i ] );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	private void readFile( String filename, String fileType ) {

		try {
			File f = new File( filename );

			if ( f.exists() ) {
				if ( fileType.equalsIgnoreCase( "Object" ) ) {
					fis = new FileInputStream( f );
					ois = new ObjectInputStream( fis );
					fileEntries = ( Vector ) ois.readObject();
					ois.close();
					fis.close();
				}
				else {
					String entry = "";
					while ( !entry.equalsIgnoreCase( "EOF" ) ) {
						entry = inputBuffer.readLine().trim();

						if ( !entry.equalsIgnoreCase( "EOF" ) &
							 !entry.equalsIgnoreCase( "File header" ) ) {
							fileEntries.add( entry );
						}
					}

					fileEntries.trimToSize();
					fileEntries = sortVector( fileEntries );
					inputBuffer.close();
					inputStream.close();
				}
			}
			else {
				System.out.println( "The file " + filename + " does not exist." );
			}
		}
		catch( Exception e ) {
			System.out.println( "There was an unknown error reading from the file." );
		}
	}

/****************************************************************/

	private void writeFile( String filename, Object o, String fileType ) {
		try {
			File f = new File( filename );

			if ( !f.exists() ) {
				f.createNewFile();
			}

			try {
				if ( fileType.equalsIgnoreCase( "Object" ) ) {
					fos = new FileOutputStream( f );
					oos = new ObjectOutputStream( fos );
					oos.writeObject( o );
					oos.flush();
					oos.close();
					fos.close();
				}
				else {
					fileEntries = sortVector( fileEntries );
					outputStream = new FileWriter( input );

					/* Write a dummy line first. */
					outputStream.write( "File header\n" );

					int counter = fileEntries.size();
					for ( int i = 0; i < counter; i++ ) {
						outputStream.write( ( String ) fileEntries.elementAt( i ) + "\n" );
					}
					outputStream.write( "EOF" );
					outputStream.close();
				}
			}
			catch( IOException ioe ) {
				ioe.printStackTrace();
				System.out.println( "There was an error writing to " +
									filename );
			}
		}
		catch( Exception e ) {
			System.out.println( "There was an unknown error writing to the file." );
		}
	}

/****************************************************************/

	/* Writes an entry to the vector,
 	   but must be in the correct format.
	*/
	public void add( String s ) {
		fileEntries.add( s );
		fileEntries = sortVector( fileEntries );
	}

/****************************************************************/

	/* Convenience method for reading from a specific file. */
	public Vector read( String targetFile, String fileType ) {
		readFile( targetFile, fileType );
		return getFileEntries();
	}

/****************************************************************/

	/* Convenience method for writing to a specific file. */
	public void write( String targetFile, Vector v, String fileType ) {
		writeFile( targetFile, v, fileType );
	}

/****************************************************************/
/*	Test harness												*/
/****************************************************************/

	public static void main( String[] args ) {
		FileSorter fs = new FileSorter( ".\\Temp.txt", "Object" );
	}
}