import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Rectangle;
import java.util.Arrays;
import java.util.Vector;
import java.util.StringTokenizer;
import NumberSelectorBox;
import Application;
import DisplayPanel;
import Marker;
import FileManagementPanel;

public class GUI extends JPanel implements ActionListener, ListSelectionListener, ItemListener, ChangeListener {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private FileManagementPanel fmp;
	private DisplayPanel dp;
	private WeaponsPanel wp;
	private Application app;
	private NumberSelectorBox nsb;
	private FileSorter fs;

	//private JPanel northPanel, southPanel;
	private JPanel northPanel;
	private JPanel westPanel, centerPanel;
	private JPanel playerButtonPanel, effectButtonPanel;
	private JTabbedPane eastPanel, southPanel;

	private JSlider zoomControl;

	private JButton resetGrid, clearDisplay;
	private JButton addFriendly, addFoe, kill, newPlayer, updatePlayer;
	private JButton addLightSource, addGrenadeEffect, addSpellEffect;
	private JButton addObstacle, addRandomObstacle;
	private JButton increaseInit, decreaseInit;

	private JComboBox lightSources, playerSizes, moveRates, weapons;
	private JComboBox spells;
	private Vector spellList = new Vector();
	private Vector lightSourceList = new Vector();
	private Vector sizeList = new Vector();
	private Vector moveRateList = new Vector();
	private Vector weaponsList = new Vector();

	private JLabel player_l, hp_l, ac_l, weapon_l, dmg_l, move_l, initiative_l, wounds_l, size_l;
	private JLabel light_l, ammo_l, melee_l, range_l, dmgMod_l;
	private JLabel spell_l;

	private JTextField player, hp, ac, weapon, dmg, initiative, wounds;
	private JTextField melee, range, dmgMod;

	private JList sequence;

	private Vector markerData = new Vector();
	private Vector justNames = new Vector();

	private int currentGridSize = 0;
	private int currentZoom = 0;
	private int currentLightCount = -100;
	private Point defaultStartingPoint = new Point( 0, 0 );

	private String playerName, playerWeapon, playerSize, playerLight, playerDmg, playerSpell;
	private int playerHP, playerAC, playerInit, playerWounds, playerMove, playerAmmo;
	private int playerMelee, playerRange, playerDmgMod;

	private static int selectedWeaponIndex = 0;
	private static final int zoomMax = 35;
	private static final int zoomMin = -15;
	private static final int zoomInterval = 5;

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public GUI( Application a ) {

		/* Setup other panel components. */
		app = a;
		nsb = new NumberSelectorBox();
		dp = new DisplayPanel( this );
		wp = new WeaponsPanel( a.getWeaponsHandler() );
		fs = a.getFileSorter();
		fmp = new FileManagementPanel( fs, this );
		currentZoom = dp.getZoom();
		currentGridSize = dp.getGridSize();
		nsb.enable();

		/* Setup vectors for the combo boxes. */
		lightSourceList = makeLightList( lightSourceList );
		weaponsList = makeWeaponsList( weaponsList );
		sizeList = makeSizeList( sizeList );
		moveRateList = makeMoveList( moveRateList );
		spellList = makeSpellList( spellList );

		/* Setup the combo boxes. */
		lightSources = makeComboBox( lightSources, lightSourceList, "None" );
		playerSizes = makeComboBox( playerSizes, sizeList, "Medium" );
		moveRates = makeComboBox( moveRates, moveRateList, "20" );
		weapons = makeComboBox( weapons, weaponsList, "None" );
		weapons.addItemListener( this );
		spells = makeComboBox( spells, spellList, "None" );

		/* Setup the slider for zooming. */
		zoomControl = new JSlider( zoomMin, zoomMax, currentZoom );
		zoomControl.setMajorTickSpacing( zoomInterval * 2 );
		zoomControl.setMinorTickSpacing( zoomInterval );
		zoomControl.setSnapToTicks( false );
		zoomControl.setPaintTicks( true );
		zoomControl.addChangeListener( this );

		/* Setup buttons. */
		resetGrid = new JButton( "Reset display" );
		clearDisplay = new JButton( "Clear display" );
		addFriendly = new JButton( "Add friendly" );
		addFoe = new JButton( "Add enemy" );
		kill = new JButton( "Kill marker" );
		newPlayer = new JButton( "New player" );
		updatePlayer = new JButton( "Update player" );
		addLightSource = new JButton( "Add light" );
		addGrenadeEffect = new JButton( "Add grenade" );
		addSpellEffect = new JButton( "Add spell" );
		addObstacle = new JButton( "Add obstacle" );
		addRandomObstacle = new JButton( "Add obstacles randomly" );
		increaseInit = new JButton( "Up" );
		decreaseInit = new JButton( "Down" );

		/* Add action listeners to the buttons. */
		addFriendly.addActionListener( this );
		addFoe.addActionListener( this );
		kill.addActionListener( this );
		clearDisplay.addActionListener( this );
		newPlayer.addActionListener( this );
		resetGrid.addActionListener( this );
		updatePlayer.addActionListener( this );
		addLightSource.addActionListener( this );
		addGrenadeEffect.addActionListener( this );
		addSpellEffect.addActionListener( this );
		addObstacle.addActionListener( this );
		addRandomObstacle.addActionListener( this );
		increaseInit.addActionListener( this );
		decreaseInit.addActionListener( this );

		/* Setup the labels. */
		player_l = new JLabel( "Player name" );
		hp_l = new JLabel( "HP" );
		ac_l = new JLabel( "AC" );
		weapon_l = new JLabel( "Active weapon" );
		dmg_l = new JLabel( "Damage" );
		initiative_l = new JLabel( "Initiative" );
		wounds_l = new JLabel( "Wounds" );
		move_l = new JLabel( "Move" );
		size_l = new JLabel( "Size" );
		light_l = new JLabel( "Light source" );
		ammo_l = new JLabel( "Ammunition" );
		melee_l = new JLabel( "Total melee bonus" );
		range_l = new JLabel( "Total range bonus" );
		dmgMod_l = new JLabel( "Total damage bonus" );
		spell_l = new JLabel( "Active spell" );

		/* Setup the textfields. */
		player = new JTextField();
		hp = new JTextField();
		ac = new JTextField();
		weapon = new JTextField();
		dmg = new JTextField();
		dmg.setText( "1d3" );
		initiative = new JTextField();
		wounds = new JTextField();
		wounds.setText( "0" );
		melee = new JTextField();
		range = new JTextField();
		dmgMod = new JTextField();

		/* Setup the list. */
		sequence = new JList();
		sequence.setFixedCellWidth( 100 );
		sequence.addListSelectionListener( this );

		/* Cater for text alignment. */
		player_l.setHorizontalAlignment( JTextField.CENTER );
		player.setHorizontalAlignment( JTextField.CENTER );
		hp_l.setHorizontalAlignment( JTextField.CENTER );
		hp.setHorizontalAlignment( JTextField.CENTER );
		ac_l.setHorizontalAlignment( JTextField.CENTER );
		ac.setHorizontalAlignment( JTextField.CENTER );
		weapon_l.setHorizontalAlignment( JTextField.CENTER );
		weapon.setHorizontalAlignment( JTextField.CENTER );
		dmg_l.setHorizontalAlignment( JTextField.CENTER );
		dmg.setHorizontalAlignment( JTextField.CENTER );
		initiative_l.setHorizontalAlignment( JTextField.CENTER );
		initiative.setHorizontalAlignment( JTextField.CENTER );
		wounds_l.setHorizontalAlignment( JTextField.CENTER );
		wounds.setHorizontalAlignment( JTextField.CENTER );
		move_l.setHorizontalAlignment( JTextField.CENTER );
		size_l.setHorizontalAlignment( JTextField.CENTER );
		light_l.setHorizontalAlignment( JTextField.CENTER );
		ammo_l.setHorizontalAlignment( JTextField.CENTER );
		melee_l.setHorizontalAlignment( JTextField.CENTER );
		melee.setHorizontalAlignment( JTextField.CENTER );
		range_l.setHorizontalAlignment( JTextField.CENTER );
		range.setHorizontalAlignment( JTextField.CENTER );
		dmgMod_l.setHorizontalAlignment( JTextField.CENTER );
		dmgMod.setHorizontalAlignment( JTextField.CENTER );
		spell_l.setHorizontalAlignment( JTextField.CENTER );

		/* Setup the look & feel of this main panel. */
		this.setBorder( new EtchedBorder() );
		this.setLayout( new BorderLayout() );

		/* Setup a subpanel to control the zoomControl. */
		JPanel zoomPanel = new JPanel();
		zoomPanel.setLayout( new FlowLayout() );
		zoomPanel.add( zoomControl );
		zoomPanel.add( resetGrid );
		zoomPanel.add( clearDisplay );

		/* Setup a subpanel to control the player-related fields. */
		JPanel playerPanel = new JPanel();
		playerPanel.setLayout( new GridLayout( 30, 1 ) );
		playerPanel.add( player_l );
		playerPanel.add( player );
		playerPanel.add( move_l );
		playerPanel.add( moveRates );
		playerPanel.add( size_l );
		playerPanel.add( playerSizes );
		playerPanel.add( hp_l );
		playerPanel.add( hp );
		playerPanel.add( wounds_l );
		playerPanel.add( wounds );
		playerPanel.add( ac_l );
		playerPanel.add( ac );
		playerPanel.add( melee_l );
		playerPanel.add( melee );
		playerPanel.add( range_l );
		playerPanel.add( range );
		playerPanel.add( weapon_l );
		playerPanel.add( weapons );
		playerPanel.add( ammo_l );
		playerPanel.add( nsb );
		playerPanel.add( dmg_l );
		playerPanel.add( dmg );
		playerPanel.add( dmgMod_l );
		playerPanel.add( dmgMod );
		playerPanel.add( initiative_l );
		playerPanel.add( initiative );
		playerPanel.add( light_l );
		playerPanel.add( lightSources );
		playerPanel.add( spell_l );
		playerPanel.add( spells );

		/* Setup a subpanel to control the increase and decrease of player initiatives. */
		JPanel initPanel = new JPanel();
		initPanel.setLayout( new GridLayout( 1, 2 ) );
		initPanel.add( increaseInit );
		initPanel.add( decreaseInit );

		/* Setup a panel to control adding player markers. */
		playerButtonPanel = new JPanel();
		playerButtonPanel.setBorder( new EtchedBorder() );
		playerButtonPanel.setLayout( new FlowLayout() );
		playerButtonPanel.add( newPlayer );
		playerButtonPanel.add( addFriendly );
		playerButtonPanel.add( addFoe );
		playerButtonPanel.add( updatePlayer );
		playerButtonPanel.add( kill );

		/* Setup a panel to control adding effects. */
		effectButtonPanel = new JPanel();
		effectButtonPanel.setBorder( new EtchedBorder() );
		effectButtonPanel.setLayout( new FlowLayout() );
		effectButtonPanel.add( addLightSource );
		effectButtonPanel.add( addGrenadeEffect );
		effectButtonPanel.add( addSpellEffect );
		effectButtonPanel.add( addObstacle );
		effectButtonPanel.add( addRandomObstacle );
		effectButtonPanel.add( kill );

		/* Add components to the panel north. */
		northPanel = new JPanel();
		northPanel.setBorder( new TitledBorder( new EtchedBorder(), "Zoom Controls" ) );
		northPanel.setLayout( new BorderLayout() );
		northPanel.add( zoomPanel, BorderLayout.WEST );
		northPanel.add( fmp, BorderLayout.EAST );

		/* Add component to the panel west. */
		JScrollPane jsp = new JScrollPane( sequence );
		westPanel = new JPanel();
		westPanel.setBorder( new TitledBorder( new EtchedBorder(), "Player List" ) );
		westPanel.setLayout( new BorderLayout() );
		westPanel.add( jsp, BorderLayout.CENTER );
		westPanel.add( initPanel, BorderLayout.SOUTH );

		/* Add components to the panel east. */
		eastPanel = new JTabbedPane( JTabbedPane.BOTTOM );
		eastPanel.setBorder( new EtchedBorder() );
		eastPanel.addTab( "Player Data", playerPanel );
		eastPanel.addTab( "Weapon Data", wp );
		eastPanel.setSelectedIndex( 0 );

		/* Add components to panel south.
		southPanel = new JPanel();
		southPanel.setBorder( new EtchedBorder() );
		southPanel.setLayout( new GridLayout( 1, 8 ) );
		southPanel.add( newPlayer );
		southPanel.add( addFriendly );
		southPanel.add( addFoe );
		southPanel.add( addLightSource );
		southPanel.add( addGrenadeEffect );
		southPanel.add( addSpellEffect );
		southPanel.add( updatePlayer );
		southPanel.add( kill ); */


		//southPanel = new JTabbedPane( JTabbedPane.BOTTOM );
		southPanel = new JTabbedPane( JTabbedPane.RIGHT );
		southPanel.addTab( "Player panel", playerButtonPanel );
		southPanel.addTab( "Effects panel", effectButtonPanel );
		southPanel.setSelectedIndex( 0 );

		/* Add components to panel center. */
		centerPanel = new JPanel();
		centerPanel.setBorder( new EtchedBorder() );
		centerPanel.setLayout( new BorderLayout() );
		centerPanel.setBackground( Color.white );
		centerPanel.add( dp, BorderLayout.CENTER );

		/* Finish this panel. */
		this.add( northPanel, BorderLayout.NORTH );
		this.add( southPanel, BorderLayout.SOUTH );
		this.add( eastPanel, BorderLayout.EAST );
		this.add( westPanel, BorderLayout.WEST );
		this.add( centerPanel, BorderLayout.CENTER );
		focus();
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public void newPlayerDoClick() {
		newPlayer.doClick();
	}

/****************************************************************/

	public Point getNextStartingPoint( Point p ) {
		int jump = currentZoom + currentGridSize;
		int x = ( int ) p.getX();
		int y = ( int ) p. getY();

		Point next = new Point( x + jump, y + jump );
		return next;
	}

/****************************************************************/

	public Vector getWeaponNames() {
		return weaponsList;
	}

/****************************************************************/

	public DisplayPanel getDisplayPanel() {
		return dp;
	}

/****************************************************************/

	/* Forces a repaint of the frame. */
	public void forcePaint() {
		repaint();
	}

/****************************************************************/

	public Vector getMarkerData() {
		markerData.trimToSize();
		return markerData;
	}

/****************************************************************/

	/* Convenience method for setting default values in the number
	 * selector box.
	 */
	public void setAmmoCount( int i ) {
		nsb.enable();
		nsb.setNumberField( i );
	}

/****************************************************************/

	/* Shifts focus immediately to the first textfield. */
	public void focus() {
		player.requestFocus();
	}

/****************************************************************/

	/* Adds the vector to the existing list data. */
	public void addListItem( Vector data ) {
		markerData = sort( data );
		justNames = sortNames( markerData );
		sequence.setListData( justNames );
	}

/****************************************************************/

	/* Deletes an item from the list,
	   given its index.
	 */
	public void deleteListItem( int i ) {
		markerData.removeElementAt( i );
		markerData.trimToSize();
		markerData = sort( markerData );
		justNames = sortNames( markerData );
		sequence.setListData( justNames );
	}

/****************************************************************/

	/* Displays a specified list item, identified by its index. */
	public void showListItem( int i ) {
		sequence.setSelectedIndex( i );
	}

/****************************************************************/

	/* Deselects all list items. */
	public void deselectList() {
		sequence.clearSelection();
	}

/****************************************************************/

	/* Refreshes the list data. */
	public void refreshListData() {
		markerData = sort( markerData );
		justNames = sortNames( markerData );
		sequence.setListData( justNames );
	}

/****************************************************************/

	/* Convenience method for creating the vector holding the
	 * items for the weapons combobox.
	 */
	public Vector makeWeaponsList( Vector target ) {
		target = wp.getWeaponNames();
		target.add( "None" );
		target.trimToSize();
		return target;
	}

/****************************************************************/

	public Vector makeLightList( Vector target ) {
		target = app.getLightHandler().getLightNames();
		target.add( "None" );
		target.trimToSize();
		return target;
	}

/****************************************************************/

	public Vector makeSpellList( Vector target ) {
		target.add( "Acid Fog" );
		target.add( "Alarm" );
		target.add( "Antilife Shell" );
		target.add( "Antimagic Field" );
		target.add( "Antiplant Shell" );
		target.add( "Bane" );
		target.add( "Blade Barrier" );
		target.add( "Blasphemy" );
		target.add( "Bless" );
		target.add( "Chain Lightning" );
		target.add( "Chaos Hammer" );
		target.add( "Circle of Death" );
		target.add( "Circle of Doom" );
		target.add( "Cloudkill" );
		target.add( "Confusion" );
		target.add( "Desecrate" );
		target.add( "Detect Scrying" );
		target.add( "Dictum" );
		target.add( "Dispel Magic" );
		target.add( "Entangle" );
		target.add( "Faerie Fire" );
		target.add( "Fireball" );
		target.add( "Flame Strike" );
		target.add( "Fog Cloud" );
		target.add( "Healing Circle" );
		target.add( "Holy Aura" );
		target.add( "Holy Smite" );
		target.add( "Holy Word" );
		target.add( "Ice Storm" );
		target.add( "Incendiary Cloud" );
		target.add( "Insect Plague" );
		target.add( "Invisibility Sphere" );
		target.add( "Leomund's Tiny Hut" );
		target.add( "Magic Circle Against ..." );
		target.add( "Mind Fog" );
		target.add( "Mordenkainen's Disjunction" );
		target.add( "Obscuring Mist" );
		target.add( "Otiluke's Freezing Sphere" );
		target.add( "Prismatic Sphere" );
		target.add( "Rainbow Pattern" );
		target.add( "Repel Vermin" );
		target.add( "Scare" );
		target.add( "Shatter" );
		target.add( "Silence" );
		target.add( "Sleep" );
		target.add( "Sleet Storm" );
		target.add( "Solid Fog" );
		target.add( "Sound Burst" );
		target.add( "Stinking Cloud" );
		target.add( "Storm of Vengeance" );
		target.add( "Summon Swarm" );
		target.add( "Teleportation Circle" );
		target.add( "Tenser's Floating Disk" );
		target.add( "Unholy Aura" );
		target.add( "Unholy Blight" );
		target.add( "Wail of the Banshee" );
		target.add( "Web" );
		target.add( "Word of Chaos" );
		target.add( "None" );
		target.trimToSize();
		return target;
	}

/****************************************************************/

	public Vector makeSizeList( Vector target ) {
		target.add( "Medium" );
		target.add( "Large, tall" );
		target.add( "Large, long" );
		target.add( "Huge, tall" );
		target.add( "Huge, long" );
		target.add( "Gargantuan, tall" );
		target.add( "Gargantuan, long" );
		target.add( "Colossal, tall" );
		target.add( "Colossal, long" );
		target.trimToSize();
		return target;
	}

/****************************************************************/

	public Vector makeMoveList( Vector target ) {
		target.add( "15" );
		target.add( "20" );
		target.add( "30" );
		target.add( "40" );
		target.add( "50" );
		target.add( "60" );
		target.add( "70" );
		target.add( "80" );
		target.add( "90" );
		target.add( "100" );
		target.trimToSize();
		return target;
	}

/****************************************************************/

	/* Convenience method for initializing a combobox
	 * with a given vector, and setting its initial selected
	 * item.
	 *
	 *.The completed result cannot be edited.
	 */
	public JComboBox makeComboBox( JComboBox target, Vector source, String selectedItem ) {
		target = new JComboBox( source );
		target.setEditable( false );
		target.setEnabled( true );
		target.setSelectedItem( selectedItem );
		return target;
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	/* Convenience method for retrieving marshalled data for saving. */
	public Vector getMarshalledData() {
		return dp.marshall();
	}

/****************************************************************/

	/* As unMarshalData, except it appends the loaded data into the
	 * currently existing vectors.
	 */
	public void appendUnMarshalData( Vector v ) {
		try {
			dp.appendUnMarshall( v );
			markerData.addAll( dp.getMarkerData() );
			refreshListData();
		}
		catch( ClassCastException cce ) {
			/* Occurs typically because changes were made to the source code
			 * files Marker, Grenade, Light, and/or Character.
			 * No known workarounds except to re-enter all the saved data.
			 */
			System.out.println( "The source code was changed!" );
		}
	}

/****************************************************************/

	/* Convenience method for unmarshalling data for loading. */
	public void unMarshalData( Vector v ) {
		try {
			dp.unmarshall( v );
			markerData = dp.getMarkerData();
			refreshListData();
		}
		catch( ClassCastException cce ) {
			/* Occurs typically because changes were made to the source code
			 * files Marker, Grenade, Light, and/or Character.
			 * No known workarounds except to re-enter all the saved data.
			 */
			System.out.println( "The source code was changed!" );
		}
	}

/****************************************************************/

	/* Convenience method for grabbing data from the various data entry fields. */
	public void getData() {
		try {
			playerHP = ( new Integer( hp.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerHP = 1;
		}

		try {
			playerAC = ( new Integer( ac.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerAC = 10;
		}

		try {
			playerInit = ( new Integer( initiative.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerInit = 0;
		}

		try {
			playerWounds = ( new Integer( wounds.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerWounds = 0;
		}

		try {
			playerMelee = ( new Integer( melee.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerMelee = 0;
		}

		try {
			playerRange = ( new Integer( range.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerRange = 0;
		}

		try {
			playerDmgMod = ( new Integer( dmgMod.getText().trim() ) ).intValue();
		}
		catch( NumberFormatException nfe ) {
			playerDmgMod = 0;
		}

		/* Items retrieved from the comboboxes are guaranteed to be Strings. */
		playerName = player.getText().trim();
		playerWeapon = ( String ) weapons.getSelectedItem();
		playerSize = ( String ) playerSizes.getSelectedItem();
		playerLight = ( String ) lightSources.getSelectedItem();
		playerSpell = ( String ) spells.getSelectedItem();
		playerMove = new Integer( ( String ) moveRates.getSelectedItem() ).intValue();
		playerDmg = dmg.getText().trim();

		/* The number selector box handles exceptions by itself,
		 * always returning an int primitive.
		 */
		playerAmmo = nsb.getNumberField();
	}

/****************************************************************/

	/* Convenience method for updating a marker's data with
	 * information retrieved from the various data entry
	 * fields.
	 */
	public void updateMarker( Marker player ) {
		player.setHP( playerHP );
		player.setAC( playerAC );
		player.setWounds( playerWounds );
		player.setMove( playerMove );
		player.setDmg( playerDmg );
		player.setWeapon( playerWeapon );
		player.setAmmo( playerAmmo );
		player.setInitiative( playerInit );
		player.setMeleeMod( playerMelee );
		player.setRangeMod( playerRange );
		player.setDmgMod( playerDmgMod );
		player.setSize( playerSize );
		player.setLightSource( playerLight );
		player.setActiveSpell( playerSpell );
		player.reCalculateAll();
	}

/****************************************************************/

	/* Convenience method for changing the internal initiative count
	 * of a marker.
	 */
	public void changeMarkerInit( float value, Marker player ) {
		float currentInit = player.getInternalInitiative();
		player.setInternalInitiative( currentInit + value );
	}

/****************************************************************/

	/* Convenience method for adding a grenade to the display,
	 * the calling method must first verify that the argument
	 * is a registered grenade or spell.
	 */
	public void addGrenade( String s ) {
		Grenade g = new Grenade( s );
		g = dp.calculateBoundingBox( defaultStartingPoint, g );
		dp.addGrenade( g );
	}

/****************************************************************/

	/* Convenience method for adding a player to the marker,
	 * the calling method must first verify that the argument
	 * is a marker.
	 */
	public void addMarker( Marker m ) {
		m = dp.calculateBoundingBox( defaultStartingPoint, m );

		int exists = playerExists( m );

		/* If a marker already exists, first remove from the list. */
		if ( exists != -1 ) {
			markerData.remove( exists );
		}

		markerData.add( m );
		addListItem( markerData );
		emptyTextFields();
	}

/****************************************************************/

	/* Returns the index iff the marker for a player already exists,
	 * -1 otherwise.
	 */
	public int playerExists( Marker m ) {
		int size = markerData.size();
		String target = m.getName();
		for ( int i = 0; i < size; i++ ) {
			String tid = ( ( Marker ) markerData.elementAt( i ) ).getName();
			if ( tid.equals( target ) ) {
				return i;
			}
		}
		return -1; // default value
	}

/****************************************************************/

	/* Sets all textfields to empty strings. */
	public void emptyTextFields() {
		player.setText( "" );
		initiative.setText( "" );
		hp.setText( "" );
		ac.setText( "" );
		dmg.setText( "1d3" );
		melee.setText( "0" );
		range.setText( "0" );
		dmgMod.setText( "0" );
		wounds.setText( "0" );
		moveRates.setSelectedItem( "20" );
		lightSources.setSelectedItem( "None" );
		playerSizes.setSelectedItem( "Medium" );
		weapons.setSelectedItem( "None" );
		spells.setSelectedItem( "None" );
		nsb.clear();
		player.requestFocus();
	}

/****************************************************************/

	/* Returns a vector of player names sorted according to the
	   original markerData vector.
	 */
	private Vector sortNames( Vector v ) {
		int counter = v.size();
		Vector result = new Vector();
		String currentEntry = "";
		for ( int i = 0; i < counter; i++ ) {
			currentEntry = ( ( Marker ) v.elementAt( i ) ).getName();
			result.add( currentEntry );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Loads up a vector of float primitives based on the
 	   original playerdata vector.
	 */
	private Vector loadInitiatives( Vector v ) {
		Vector result = new Vector();
		int counter = v.size();

		for ( int i = 0; i < counter; i++ ) {
			float value = ( ( Marker ) v.elementAt( i ) ).getInternalInitiative();
			//int value = ( ( Marker ) v.elementAt( i ) ).getInitiative();
			result.add( new Float( value ) );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Sorts a vector numerically in descending order. */
	private Vector sortVector( Vector v ) {
		Vector temp = loadInitiatives( v );

		int size = temp.size();
		float[] initiatives = new float[ size ];
		for ( int i = 0; i < size; i++ ) {
			initiatives[ i ] = ( ( Float ) temp.elementAt( i ) ).floatValue();
		}

		Object[] toBeSorted = temp.toArray();
		Arrays.sort( toBeSorted );

		/* Assert: Array now sorted in ascending order. */
		Vector result = new Vector();
		int counter = toBeSorted.length - 1;
		for ( int i = counter; i != -1; i-- ) {
			result.add( toBeSorted[ i ] );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Returns a list of element with initiative values matching the
	   argument.
	 */
	public Vector getElements( float target, Vector source ) {
		Vector result = new Vector();
		int counter = source.size();
		Marker currentEntry;
		float currentInitiative = ( float ) 0;

		for ( int k = 0; k < counter; k++ ) {
			currentEntry = ( Marker ) source.elementAt( k );
			currentInitiative = currentEntry.getInternalInitiative();
			//currentInitiative = currentEntry.getInitiative();

			if ( currentInitiative == target ) {
				result.add( currentEntry );
			}
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Re-organizes the original markerData vector according to
	   the sorted initiatives vectors.
	 */
	private Vector sort( Vector v ) {
		/* Get the vector of sorted player initiatives. */
		Vector temp = sortVector( v );
		Vector result = new Vector();
		int matchCount = 0;

		int counter = temp.size();
		for ( int i = 0; i < counter; i++ ) {
			float currentValue = ( ( Float ) temp.elementAt( i ) ).floatValue();
			//int currentValue = ( ( Integer ) temp.elementAt( i ) ).intValue();
			Vector retrievedMarkers = getElements( currentValue, v );

			for ( int j = 0; j < retrievedMarkers.size(); j++ ) {
				Marker tempMarker = ( Marker ) retrievedMarkers.elementAt( j );

				if ( !result.contains( tempMarker ) ) {
					result.add( tempMarker  );
				}
			}
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/
/*	Action Listeners											*/
/****************************************************************/

	public void actionPerformed( ActionEvent ae ) {

		/* Calculate the starting point for placing the marker.
		 * If there is already a marker there,
		 * get a new starting point to the south-east of it.
		 */
		if ( dp.markerAt( defaultStartingPoint ) | dp.lightSourceAt( defaultStartingPoint ) |
			dp.grenadeAt( defaultStartingPoint ) | dp.obstacleAt( defaultStartingPoint ) ) {
			defaultStartingPoint = getNextStartingPoint( defaultStartingPoint );
		}
		else {
			defaultStartingPoint = new Point();
		}

		/* The rest of the action listeners for the remaining buttons. */
		if ( ae.getSource() == resetGrid ) {
			zoomControl.setValue( 0 );
			dp.resetZoom();
			dp.resetPlayerStatus();
		}
		else if ( ae.getSource() == clearDisplay ) {
			emptyTextFields();
			justNames = new Vector();
			markerData = new Vector();
			sequence.setListData( justNames );
			currentLightCount = -100;
			dp.clearAll();
		}
		else if ( ae.getSource() == newPlayer ) {
			emptyTextFields();
			sequence.clearSelection();
		}
		else if ( ae.getSource() == increaseInit ) {
			int index = sequence.getSelectedIndex();

			if ( index != -1 ) {
				Marker player = ( Marker ) markerData.elementAt( index );
				changeMarkerInit( ( float ) 0.1, player );
				refreshListData();
			}
		}
		else if ( ae.getSource() == decreaseInit ) {
			int index = sequence.getSelectedIndex();

			if ( index != -1 ) {
				Marker player = ( Marker ) markerData.elementAt( index );
				changeMarkerInit( ( float ) -0.1, player );
				refreshListData();
			}
		}
		else if ( ae.getSource() == kill ) {
			try {
				emptyTextFields();
				int selectedIndex = sequence.getSelectedIndex();
				deleteListItem( selectedIndex );
			}
			catch( ArrayIndexOutOfBoundsException aiooe ) {
				dp.killLightMode();
				dp.killGrenadeMode();
				dp.killObstacleMode();
			}
		}
		else if ( ae.getSource() == addLightSource ) {
			String selectedItem = ( String ) lightSources.getSelectedItem();
			Light l = new Light( selectedItem );
			l = dp.calculateBoundingBox( defaultStartingPoint, l );
			dp.addLightSource( l );
		}
		else if ( ae.getSource() == addGrenadeEffect ) {

			String selectedItem = ( String ) weapons.getSelectedItem();

			/* Make sure that a grenade has been selected before further processing. */
			if ( app.getGrenadeHandler().isGrenade( selectedItem ) ) {
				addGrenade( selectedItem );
			}
		}
		else if ( ae.getSource() == addSpellEffect ) {
			String selectedItem = ( String ) spells.getSelectedItem();

			/* Make sure that a spell has been selected. */
			if ( !selectedItem.equalsIgnoreCase( "None" ) ) {
				addGrenade( selectedItem );
			}
		}
		else if ( ae.getSource() == addObstacle ) {
			Obstacle o = new Obstacle();
			o = dp.calculateBoundingBox( defaultStartingPoint, o );
			dp.addObstacle( o );
		}
		else if ( ae.getSource() == addRandomObstacle ) {
			for ( int i = 0; i < 10; i++ ) {
				int randX = ( int ) ( Math.random() * ( double ) DisplayPanel.maxX );
				int randY = ( int ) ( Math.random() * ( double ) DisplayPanel.maxY );
				Point p = new Point( randX, randY );

				Obstacle o = new Obstacle();
				o = dp.calculateBoundingBox( p, o );
				dp.addObstacle( o );
			}
		}
		else if ( ae.getSource() == updatePlayer ) {

			int index = sequence.getSelectedIndex();

			if ( index != -1 ) {
				getData();
				Marker player = ( Marker ) markerData.elementAt( index );
				updateMarker( player );
				refreshListData();
			}
		}
		else if ( ae.getSource() == addFriendly ) {

			getData();

			Marker player = new Marker( playerHP, playerAC, playerInit, playerWounds, playerMove,
										playerName, playerWeapon, playerAmmo, playerDmg, playerSize,
										playerLight, playerMelee, playerRange, playerDmgMod,
										playerSpell, true );
			addMarker( player );
		}
		else if ( ae.getSource() == addFoe ) {

			getData();

			Marker player = new Marker( playerHP, playerAC, playerInit, playerWounds, playerMove,
										playerName, playerWeapon, playerAmmo, playerDmg, playerSize,
										playerLight, playerMelee, playerRange, playerDmgMod,
										playerSpell, false );
			addMarker( player );
		}
		dp.refreshData();
		forcePaint();
		focus();
	}

/****************************************************************/
/*	ListSelectionListeners										*/
/****************************************************************/

	public void valueChanged( ListSelectionEvent lse ) {
		try {
			int index = sequence.getSelectedIndex();
			nsb.enable();

			Marker currentEntry = ( Marker ) markerData.elementAt( index );

			/* Show the data of the marker. */
			player.setText( currentEntry.getName() );
			initiative.setText( currentEntry.getInitiative() + "" );
			hp.setText( currentEntry.getHP() + "" );
			ac.setText( currentEntry.getAC() + "" );
			wounds.setText( currentEntry.getWounds() + "" );
			melee.setText( currentEntry.getMeleeMod() + "" );
			range.setText( currentEntry.getRangeMod() + "" );
			dmgMod.setText( currentEntry.getDmgMod() + "" );
			dmg.setText( currentEntry.getDmg() );
			nsb.setNumberField( currentEntry.getAmmo() + "" );
			weapons.setSelectedItem( currentEntry.getWeapon() + "" );
			moveRates.setSelectedItem( currentEntry.getMove() + "" );
			playerSizes.setSelectedItem( currentEntry.getSize() );
			lightSources.setSelectedItem( currentEntry.getLightSource() );
			spells.setSelectedItem( currentEntry.getActiveSpell() );
		}
		catch( ArrayIndexOutOfBoundsException aiobe ) {}
	}

/****************************************************************/
/*	Item Listeners												*/
/****************************************************************/

	public void itemStateChanged( ItemEvent ie ) {
		String currentSelected;
		String currentDmg = "";
		int selectedIndex;
		dmg.setText( "" );

		if ( ie.getItemSelectable() == weapons ) {

			currentSelected = ( String ) weapons.getSelectedItem();
			selectedIndex = weapons.getSelectedIndex();

			if ( currentSelected.equalsIgnoreCase( "none" ) ) {
				dmg.setText( "1d3" );
			}
			else {
				currentDmg = app.getWeaponsHandler().getDmg( currentSelected );
				dmg.setText( currentDmg );
			}

			if ( selectedIndex != -1 ) {
				wp.displayItemAt( selectedIndex );
			}
		}
	}

/****************************************************************/
/*	Change Listeners											*/
/****************************************************************/

	public void stateChanged( ChangeEvent ce ) {
		currentZoom = zoomControl.getValue();
		dp.setZoom( currentZoom );
	}
}