import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Vector;

public class JumpCalculator {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private final double staticMod = 10.0;

	private final double runJumpMin = 5.0;
	private final double standJumpMin = 3.0;
	private final double runHiJumpMin = 2.0;
	private final double standHiJumpMin = 2.0;
	private final double jumpBackMin = 1.0;

	private final double runJumpMaxMod = 6.0;
	private final double standJumpMaxMod = 2.0;
	private final double runHiJumpMaxMod = 1.5;
	private final double standHiJumpMaxMod = 1.0;
	private final double jumpBackMaxMod = 1.0;

	private final double runJumpDistMod = 1.0;
	private final double standJumpDistMod = 2.0;
	private final double runHiJumpDistMod = 4.0;
	private final double standHiJumpDistMod = 8.0;
	private final double jumpBackDistMod = 8.0;

	private final double baseSpeed = 30.0;

/****************************************************************/
/*	Constructor													*/
/****************************************************************/

	// Use default constructor

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	/* Returns a measurement in feet/inches to solely inches. */
	public double convertHeightToInches( double feet, double inch ) {
		return ( feet * 12.0 ) + inch;
	}

/****************************************************************/

	/* Returns a height value constructed from inches. */
	public double[] convertInchesToHeight( double i ) {
		double[] result = new double[ 2 ];
		result[ 0 ] = i/12;
		result[ 1 ] = i%12;
		return result;
	}

/****************************************************************/

	/* Returns the max distance of a running jump, based on the
	 * player's height.
	 */
	public double calculateRunningJumpMax( double playerHeight ) {
		return playerHeight * runJumpMaxMod;
	}

/****************************************************************/

	public double calculateStandingJumpMax( double playerHeight ) {
		return playerHeight * standJumpMaxMod;
	}

/****************************************************************/

	public double calculateRunningHighJumpMax( double playerHeight ) {
		return playerHeight * runHiJumpMaxMod;
	}

/****************************************************************/

	public double calculateStandingHighJumpMax( double playerHeight ) {
		return playerHeight * standHiJumpMaxMod;
	}

/****************************************************************/

	public double calculateJumpBackMax( double playerHeight ) {
		return playerHeight * jumpBackMaxMod;
	}

/****************************************************************/

	/* Returns the distance jumped using a running jump,
	 * given a jump check roll and the player's height
	 * in inches.
	 */
	public double calculateRunningJump( double dieRoll, double playerHeight, double playerSpeed ) {
		double modifier = dieRoll - staticMod;
		double speedMod = ( playerSpeed - baseSpeed ) / baseSpeed;
		double max = playerHeight * runJumpMaxMod;
		double result = runJumpMin + ( modifier / runJumpDistMod );

		if ( result <= runJumpMin ) {
			return runJumpMin;
		}
		else if ( result > max ) {
			return max;
		}
		else {
			return ( ( result * speedMod ) + result ) * 12.0;
		}
	}

/****************************************************************/

	/* Same as above, except calculated for a standing jump. */
	public double calculateStandingJump( double dieRoll, double playerHeight, double playerSpeed ) {
		double modifier = dieRoll - staticMod;
		double speedMod = ( playerSpeed - baseSpeed ) / baseSpeed;
		double max = playerHeight * standJumpMaxMod;
		double result = standJumpMin + ( modifier / standJumpDistMod );

		if ( result <= standJumpMin ) {
			return standJumpMin;
		}
		else if ( result > max ) {
			return max;
		}
		else {
			return ( ( result * speedMod ) + result ) * 12.0;
		}
	}

/****************************************************************/

	/* Same as above, except calculated for a running high jump. */
	public double calculateRunningHighJump( double dieRoll, double playerHeight, double playerSpeed ) {
		double modifier = dieRoll - staticMod;
		double speedMod = ( playerSpeed - baseSpeed ) / baseSpeed;
		double max = playerHeight * runHiJumpMaxMod;
		double result = runHiJumpMin + ( modifier / runHiJumpDistMod );

		if ( result <= runHiJumpMin ) {
			return runHiJumpMin;
		}
		else if ( result > max ) {
			return max;
		}
		else {
			return ( ( result * speedMod ) + result ) * 12.0;
		}
	}

/****************************************************************/

	/* Same as above, except calculated for a standing high jump. */
	public double calculateStandingHighJump( double dieRoll, double playerHeight, double playerSpeed ) {
		double modifier = dieRoll - staticMod;
		double speedMod = ( playerSpeed - baseSpeed ) / baseSpeed;
		double max = playerHeight * standHiJumpMaxMod;
		double result = standHiJumpMin + ( modifier / standHiJumpDistMod );

		if ( modifier <= standHiJumpMin ) {
			return standHiJumpMin;
		}
		else if ( result > max ) {
			return max;
		}
		else {
			return ( ( result * speedMod ) + result ) * 12.0;
		}
	}

/****************************************************************/

	/* Same as above, except calculated for jumping backwards. */
	public double calculateJumpBack( double dieRoll, double playerHeight, double playerSpeed ) {
		double modifier = dieRoll - staticMod;
		double speedMod = ( playerSpeed - baseSpeed ) / baseSpeed;
		double max = playerHeight * jumpBackMaxMod;
		double result = jumpBackMin + ( modifier / jumpBackDistMod );

		if ( modifier <= jumpBackMin ) {
			return jumpBackMin;
		}
		else if ( result > max ) {
			return max;
		}
		else {
			return ( ( result * speedMod ) + result ) * 12.0;
		}
	}
}