import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.EtchedBorder;
import java.util.Vector;
import HeightControl_GUI;
import JumpSelector_GUI;
import JumpCalculator;

public class JumpCalculator_GUI extends JPanel implements ActionListener {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private HeightControl_GUI heightControl, distanceControl, maxDistanceControl;
	private JumpSelector_GUI js_gui;
	private JumpCalculator jc;

	private JLabel height_l, maxDistance_l,	distanceJumped_l, diceRoll_l, baseSpeed_l;
	private JTextField diceRoll, baseSpeed;
	private JButton calculate, clear;

	private double height_feet_field, height_inch_field, height_field;
	private double diceRoll_field, baseSpeed_field;
	private double maxDistance_field, distanceJumped_field;

/****************************************************************/
/*	Constructor													*/
/****************************************************************/

	public JumpCalculator_GUI() {

		heightControl = new HeightControl_GUI();
		heightControl.enable();
		distanceControl = new HeightControl_GUI();
		distanceControl.disable();
		maxDistanceControl = new HeightControl_GUI();
		maxDistanceControl.disable();

		js_gui = new JumpSelector_GUI();
		jc = new JumpCalculator();

		/* Setup all labels. */
		height_l = new JLabel( "Player height" );
		maxDistance_l = new JLabel( "Max possible jump distance" );
		distanceJumped_l = new JLabel( "Total distance jumped" );
		diceRoll_l = new JLabel( "Jump check roll" );
		baseSpeed_l = new JLabel( "Base speed of player" );

		/* Setup all text fields. */
		diceRoll = new JTextField();
		baseSpeed = new JTextField( "30" );

		/* Setup all buttons. */
		calculate = new JButton( "Calculate jump" );
		calculate.addActionListener( this );
		clear = new JButton( "Clear" );
		clear.addActionListener( this );

		/* Setup a panel to hold the data entry items. */
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout( new GridLayout( 5, 2 ) );
		centerPanel.add( height_l );
		centerPanel.add( heightControl );
		centerPanel.add( diceRoll_l );
		centerPanel.add( diceRoll );
		centerPanel.add( baseSpeed_l );
		centerPanel.add( baseSpeed );
		centerPanel.add( maxDistance_l );
		centerPanel.add( maxDistanceControl );
		centerPanel.add( distanceJumped_l );
		centerPanel.add( distanceControl );

		/* Setup a panel to hold the buttons. */
		JPanel southPanel = new JPanel();
		southPanel.setLayout( new GridLayout( 2, 1 ) );
		southPanel.add( calculate );
		southPanel.add( clear );

		this.setBorder( new EtchedBorder() );
		this.setLayout( new BorderLayout() );
		this.add( js_gui, BorderLayout.NORTH );
		this.add( centerPanel, BorderLayout.CENTER );
		this.add( southPanel, BorderLayout.SOUTH );
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public void clear() {
		js_gui.clear();
		heightControl.clear();
		distanceControl.clear();
		maxDistanceControl.clear();
		diceRoll.setText( "" );
		baseSpeed.setText( "" );
	}

/****************************************************************/

	/* Convenience method for grabbing data from the text fields. */
	private void getData() {
		height_feet_field = ( double ) heightControl.getFeetMeasurement();
		height_inch_field = ( double ) heightControl.getInchMeasurement();

		try {
			diceRoll_field = ( new Double( diceRoll.getText() ) ).doubleValue();
		}
		catch( NumberFormatException nfe ) {
			diceRoll_field = 0.0;
		}

		try {
			baseSpeed_field = ( new Double( baseSpeed.getText() ) ).doubleValue();
		}
		catch( NumberFormatException nfe ) {
			baseSpeed_field = 30.0;;
		}
	}

/****************************************************************/

	/* Convenience method for setting the distance jumped. */
	public void setDistanceJumped( double[] distance ) {
		int feet = ( int ) distance[ 0 ];
		int inch = ( int ) distance[ 1 ];
		distanceControl.setFeetMeasurement( feet );
		distanceControl.setInchMeasurement( inch );
	}

/****************************************************************/

	/* Convenience method for setting the maximum jumpable distance. */
	public void setMaxDistance( double[] distance ) {
		int feet = ( int ) distance[ 0 ];
		int inch = ( int ) distance[ 1 ];
		maxDistanceControl.setFeetMeasurement( feet );
		maxDistanceControl.setInchMeasurement( inch );
	}

/****************************************************************/
/*	Action Listeners											*/
/****************************************************************/

	public void actionPerformed( ActionEvent ae ) {
		if ( ae.getSource() == clear ) {
			clear();
		}
		else {
			String jumpType = js_gui.getSelectedButton();
			getData();
			height_field = jc.convertHeightToInches( height_feet_field, height_inch_field );

			if ( jumpType.equalsIgnoreCase( "Running jump" ) ) {
				distanceJumped_field = jc.calculateRunningJump( diceRoll_field, height_field, baseSpeed_field );
				maxDistance_field = jc.calculateRunningJumpMax( height_field );
			}
			else if ( jumpType.equalsIgnoreCase( "Standing jump" ) ) {
				distanceJumped_field = jc.calculateStandingJump( diceRoll_field, height_field, baseSpeed_field );
				maxDistance_field = jc.calculateStandingJumpMax( height_field );
			}
			else if ( jumpType.equalsIgnoreCase( "Running high jump" ) ) {
				distanceJumped_field = jc.calculateRunningHighJump( diceRoll_field, height_field, baseSpeed_field );
				maxDistance_field = jc.calculateRunningHighJumpMax( height_field );
			}
			else if ( jumpType.equalsIgnoreCase( "Standing high jump" ) ) {
				distanceJumped_field = jc.calculateStandingHighJump( diceRoll_field, height_field, baseSpeed_field );
				maxDistance_field = jc.calculateStandingHighJumpMax( height_field );
			}
			else {
				distanceJumped_field = jc.calculateJumpBack( diceRoll_field, height_field, baseSpeed_field );
				maxDistance_field = jc.calculateJumpBackMax( height_field );
			}
			setDistanceJumped( jc.convertInchesToHeight( distanceJumped_field ) );
			setMaxDistance( jc.convertInchesToHeight( maxDistance_field ) );
		}
	}
}