import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;
import java.util.Arrays;
import Marker;

public class ListPanel extends JPanel implements ListSelectionListener, ActionListener {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private JButton increment, decrement;
	private JList list;
	private JScrollPane jsp;
	private Vector listData, markerData;

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public ListPanel() {
		/* Setup the list. */
		listData = new Vector();
		list = new JList( listData );
		list.setFixedCellWidth( 100 );
		list.addListSelectionListener( this );

		/* Setup the scroll pane. */
		jsp = new JScrollPane( list );

		/* Setup the look and feel of this panel. */
		this.setLayout( new BorderLayout() );
		this.setBorder( new TitledBorder( new EtchedBorder(), "Player List" ) );

		/* Fill in the contents of this panel. */
		this.add( jsp, BorderLayout.CENTER );
		this.add( makeButtonPanel(), BorderLayout.SOUTH );
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public JList getList() {
		return list;
	}

	public Vector getListData() {
		return listData;
	}

	public void setList( JList jl ) {
		list = jl;
	}

	public void setListData( Vector v ) {
		listData = v;
	}

	public void clearSelection() {
		list.clearSelection();
	}

	public void clearList() {
		listData = new Vector();
		list.setListData( listData );
	}

	public void addListItem( Vector v ) {
		markerData = sort( v );
		listData = sortNames( markerData );
		list.setListData( listData );
	}

	public void deleteListItem( int index ) {
		markerData.removeElementAt( index );
		markerData.trimToSize();
		addListItem( markerData );
	}

	public void refreshListData() {
		markerData = sort( markerData );
		listData = sortNames( markerData );
		list.setListData( listData );
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	private JPanel makeButtonPanel() {
		increment = new JButton( "Up" );
		decrement = new JButton( "Down" );
		increment.addActionListener( this );
		decrement.addActionListener( this );

		JPanel result = new JPanel();
		result.setLayout( new GridLayout( 1, 2 ) );
		result.add( increment );
		result.add( decrement );
		return result;
	}

/****************************************************************/

	/* Returns a vector of player names sorted according to the
	   original markerData vector.
	 */
	private Vector sortNames( Vector v ) {
		int counter = v.size();
		Vector result = new Vector();
		String currentEntry = "";
		for ( int i = 0; i < counter; i++ ) {
			currentEntry = ( ( Marker ) v.elementAt( i ) ).getName();
			result.add( currentEntry );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Loads up a vector of float primitives based on the
 	   original playerdata vector.
	 */
	private Vector loadInitiatives( Vector v ) {
		Vector result = new Vector();
		int counter = v.size();

		for ( int i = 0; i < counter; i++ ) {
			float value = ( ( Marker ) v.elementAt( i ) ).getInternalInitiative();
			//int value = ( ( Marker ) v.elementAt( i ) ).getInitiative();
			result.add( new Float( value ) );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Sorts a vector numerically in descending order. */
	private Vector sortVector( Vector v ) {
		Vector temp = loadInitiatives( v );

		int size = temp.size();
		float[] initiatives = new float[ size ];
		for ( int i = 0; i < size; i++ ) {
			initiatives[ i ] = ( ( Float ) temp.elementAt( i ) ).floatValue();
		}

		Object[] toBeSorted = temp.toArray();
		Arrays.sort( toBeSorted );

		/* Assert: Array now sorted in ascending order. */
		Vector result = new Vector();
		int counter = toBeSorted.length - 1;
		for ( int i = counter; i != -1; i-- ) {
			result.add( toBeSorted[ i ] );
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Returns a list of element with initiative values matching the
	   argument.
	 */
	public Vector getElements( float target, Vector source ) {
		Vector result = new Vector();
		int counter = source.size();
		Marker currentEntry;
		float currentInitiative = ( float ) 0;

		for ( int k = 0; k < counter; k++ ) {
			currentEntry = ( Marker ) source.elementAt( k );
			currentInitiative = currentEntry.getInternalInitiative();
			//currentInitiative = currentEntry.getInitiative();

			if ( currentInitiative == target ) {
				result.add( currentEntry );
			}
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/

	/* Re-organizes the original markerData vector according to
	   the sorted initiatives vectors.
	 */
	private Vector sort( Vector v ) {
		/* Get the vector of sorted player initiatives. */
		Vector temp = sortVector( v );
		Vector result = new Vector();
		int matchCount = 0;

		int counter = temp.size();
		for ( int i = 0; i < counter; i++ ) {
			float currentValue = ( ( Float ) temp.elementAt( i ) ).floatValue();
			//int currentValue = ( ( Integer ) temp.elementAt( i ) ).intValue();
			Vector retrievedMarkers = getElements( currentValue, v );

			for ( int j = 0; j < retrievedMarkers.size(); j++ ) {
				Marker tempMarker = ( Marker ) retrievedMarkers.elementAt( j );

				if ( !result.contains( tempMarker ) ) {
					result.add( tempMarker  );
				}
			}
		}
		result.trimToSize();
		return result;
	}

/****************************************************************/
/*	Action Listeners											*/
/****************************************************************/

	public void actionPerformed( ActionEvent ae ) {
		int index = list.getSelectedIndex();

		if ( index != -1 ) {
			if ( ae.getSource() == increment ) {

			}
			else {

			}
		}
	}

/****************************************************************/
/*	List Selection Listeners									*/
/****************************************************************/

	public void valueChanged( ListSelectionEvent lse ) {

	}
}