import java.util.Vector;
import java.util.StringTokenizer;
import FileSorter;

public class SpellHandler {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private Vector spellVector = new Vector();
	private Vector spellNames = new Vector();
	private FileSorter fs;

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public SpellHandler( FileSorter f ) {
		fs = f;
		Vector temp = fs.read( Application.spellFile, "Text" );
		loadSpells( temp );
		loadSpellNames( spellVector );
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public Vector getSpells() {
		return spellVector;
	}

	public Vector getSpellNames() {
		return spellNames;
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	/* Returns true iff the argument is a registered spell. */
	public boolean isSpell( String s ) {
		if ( spellNames.contains( s ) ) {
			return true;
		}
		return false;
	}

/****************************************************************/

	/* Creates a vector of spell objects from the raw format
	 * retrieved from file.
	 */
	private void loadSpells( Vector v ) {

		int size = v.size();
		StringTokenizer st;

		for ( int i = 0; i < size; i++ ) {
			String currentEntry = ( ( String ) v.elementAt( i ) ).trim();

			if ( !currentEntry.equals( "" ) & currentEntry != null ) {

				st = new StringTokenizer( currentEntry, "\t" );

				String name = st.nextToken();
				//System.out.println( "Name: " + name );

				String effect = st.nextToken();
				//System.out.println( "Effect: " + effect );

				String school = st.nextToken();
				//System.out.println( "School: " + school );

				String level = st.nextToken();
				//System.out.println( "Level: " + level );

				String components = st.nextToken();
				//System.out.println( "Components: " + components );

				String castTime = st.nextToken();
				//System.out.println( "Casting time: " + castTime );

				String range = st.nextToken();
				//System.out.println( "Range: " + range );

				String duration = st.nextToken();
				//System.out.println( "Duration: " + duration );

				String save = st.nextToken();
				//System.out.println( "Save: " + save );

				String resist = st.nextToken();
				//System.out.println( "Resistance: " + resist );
				//System.out.println();

				Spell s = new Spell( name, level, school, components, castTime, range, effect,
						 			 duration, save, resist );
				spellVector.add( s );
			}
		}
		spellVector.trimToSize();
	}

/****************************************************************/

	/* Strips the spell names from a vector of spell objects,
	 * and loads it into an array.
	 */
	private void loadSpellNames( Vector v ) {
		int size = v.size();
		for ( int i = 0; i < size; i++ ) {
			Spell temp = ( Spell ) spellVector.elementAt( i );
			spellNames.add( temp.getSpellName() );
		}
		spellNames.trimToSize();
	}

/****************************************************************/

	/* Returns the specified spell. */
	private Spell getSpell( String s ) {
		int index = getIndexOfSpell( s );
		if ( index != -1 ) {
			return ( Spell ) spellVector.elementAt( index );
		}
		return null;
	}

/****************************************************************/

	/* Returns the index of the specified weapon. */
	private int getIndexOfSpell( String s ) {
		if ( spellNames.contains( s ) ) {
			return spellNames.indexOf( s );
		}
		return -1;
	}

/****************************************************************/

	/* Returns the spell effect for a specific spell. */
	public String getSpellEffect( String s ) {
		Spell temp = getSpell( s );
		return temp.getSpellEffect();
	}
}