import java.util.Vector;
import java.util.StringTokenizer;
import FileSorter;

public class WeaponsHandler {

/****************************************************************/
/*	Declarations												*/
/****************************************************************/

	private Vector weaponsVector = new Vector();
	private Vector weaponsNames = new Vector();
	private FileSorter fs;

/****************************************************************/
/*	Constructors												*/
/****************************************************************/

	public WeaponsHandler( FileSorter a ) {
		fs = a;
		Vector temp = fs.read( Application.weaponsFile, "Text" );
		loadWeapons( temp );
		loadWeaponsNames( weaponsVector );
	}

/****************************************************************/
/*	Accessors													*/
/****************************************************************/

	public Vector getWeapons() {
		return weaponsVector;
	}

	public Vector getWeaponsNames() {
		return weaponsNames;
	}

/****************************************************************/
/*	Methods														*/
/****************************************************************/

	/* Creates a vector of Weapon objects from the raw format
	 * retrieved from file.
	 */
	private void loadWeapons( Vector v ) {

		int size = v.size();
		StringTokenizer st;

		for ( int i = 0; i < size; i++ ) {
			String currentEntry = ( ( String ) v.elementAt( i ) ).trim();

			if ( !currentEntry.equals( "" ) & currentEntry != null ) {

				st = new StringTokenizer( currentEntry, "\t" );

				String name = st.nextToken();
				//System.out.println( "Weapon name: " + name );

				String dmg = st.nextToken();
				//System.out.println( "Weapon dmg: " + dmg );

				String critThreat = st.nextToken();
				//System.out.println( "Weapon crit range: " + critThreat );

				String critical = st.nextToken();
				//System.out.println( "Weapon crit dmg: " + critical );

				String dmgType = st.nextToken();
				//System.out.println( "Weapon dmy type: " + dmgType + "\n" );

				Weapon currentWeapon = new Weapon( name, dmg, critThreat, critical, dmgType );
				weaponsVector.add( currentWeapon );
			}
		}
		weaponsVector.trimToSize();
	}

/****************************************************************/

	/* Strips the weapons names from a vector of weapons,
	 * and loads it into an array.
	 */
	private void loadWeaponsNames( Vector v ) {
		int size = v.size();
		for ( int i = 0; i < size; i++ ) {
			Weapon temp = ( Weapon ) weaponsVector.elementAt( i );
			weaponsNames.add( temp.getName() );
		}
		weaponsNames.trimToSize();
	}

/****************************************************************/

	/* Returns the specified weapon. */
	private Weapon getWeapon( String s ) {
		int index = getIndexOfWeapon( s );
		if ( index != -1 ) {
			return ( Weapon ) weaponsVector.elementAt( index );
		}
		return null;
	}

/****************************************************************/

	/* Returns the index of the specified weapon. */
	private int getIndexOfWeapon( String s ) {
		if ( weaponsNames.contains( s ) ) {
			return weaponsNames.indexOf( s );
		}
		return -1;
	}

/****************************************************************/

	/* Returns the damage caused by a weapon. */
	public String getDmg( String s ) {
		Weapon currentWeapon = getWeapon( s );
		return currentWeapon.getDmg();
	}
}